package com.example.moventaa.bean;

import com.example.moventaa.bean.PMF;

import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiNamespace;
import com.google.api.server.spi.response.CollectionResponse;
import com.google.appengine.api.datastore.Cursor;
import com.google.appengine.datanucleus.query.JDOCursorHelper;

import java.util.HashMap;
import java.util.List;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

@Api(name = "stocksolicitadoendpoint", namespace = @ApiNamespace(ownerDomain = "example.com", ownerName = "example.com", packagePath = "moventaa.bean"))
public class StockSolicitadoEndpoint {

	/**
	 * This method lists all the entities inserted in datastore.
	 * It uses HTTP GET method and paging support.
	 *
	 * @return A CollectionResponse class containing the list of all entities
	 * persisted and a cursor to the next page.
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "listStockSolicitado")
	public CollectionResponse<StockSolicitado> listStockSolicitado(
			@Nullable @Named("cursor") String cursorString,
			@Nullable @Named("limit") Integer limit) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<StockSolicitado> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(StockSolicitado.class);
			if (cursorString != null && cursorString != "") {
				cursor = Cursor.fromWebSafeString(cursorString);
				HashMap<String, Object> extensionMap = new HashMap<String, Object>();
				extensionMap.put(JDOCursorHelper.CURSOR_EXTENSION, cursor);
				query.setExtensions(extensionMap);
			}

			if (limit != null) {
				query.setRange(0, limit);
			}

			execute = (List<StockSolicitado>) query.execute();
			cursor = JDOCursorHelper.getCursor(execute);
			if (cursor != null)
				cursorString = cursor.toWebSafeString();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (StockSolicitado obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<StockSolicitado> builder().setItems(execute)
				.setNextPageToken(cursorString).build();
	}

	/**
	 * This method gets the entity having primary key id. It uses HTTP GET method.
	 *
	 * @param id the primary key of the java bean.
	 * @return The entity with primary key id.
	 */
	@ApiMethod(name = "getStockSolicitado")
	public StockSolicitado getStockSolicitado(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		StockSolicitado stocksolicitado = null;
		try {
			stocksolicitado = mgr.getObjectById(StockSolicitado.class, id);
		} finally {
			mgr.close();
		}
		return stocksolicitado;
	}

	/**
	 * This inserts a new entity into App Engine datastore. If the entity already
	 * exists in the datastore, an exception is thrown.
	 * It uses HTTP POST method.
	 *
	 * @param stocksolicitado the entity to be inserted.
	 * @return The inserted entity.
	 */
	@ApiMethod(name = "insertStockSolicitado")
	public StockSolicitado insertStockSolicitado(StockSolicitado stocksolicitado) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			//if (containsStockSolicitado(stocksolicitado)) {
			//	throw new EntityExistsException("Object already exists");
			//}
			mgr.makePersistent(stocksolicitado);
		} finally {
			mgr.close();
		}
		return stocksolicitado;
	}

	/**
	 * This method is used for updating an existing entity. If the entity does not
	 * exist in the datastore, an exception is thrown.
	 * It uses HTTP PUT method.
	 *
	 * @param stocksolicitado the entity to be updated.
	 * @return The updated entity.
	 */
	@ApiMethod(name = "updateStockSolicitado")
	public StockSolicitado updateStockSolicitado(StockSolicitado stocksolicitado) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (!containsStockSolicitado(stocksolicitado)) {
				throw new EntityNotFoundException("Object does not exist");
			}
			mgr.makePersistent(stocksolicitado);
		} finally {
			mgr.close();
		}
		return stocksolicitado;
	}

	/**
	 * This method removes the entity with primary key id.
	 * It uses HTTP DELETE method.
	 *
	 * @param id the primary key of the entity to be deleted.
	 */
	@ApiMethod(name = "removeStockSolicitado")
	public void removeStockSolicitado(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			StockSolicitado stocksolicitado = mgr.getObjectById(
					StockSolicitado.class, id);
			mgr.deletePersistent(stocksolicitado);
		} finally {
			mgr.close();
		}
	}

	private boolean containsStockSolicitado(StockSolicitado stocksolicitado) {
		PersistenceManager mgr = getPersistenceManager();
		boolean contains = true;
		try {
			mgr.getObjectById(StockSolicitado.class, stocksolicitado.getKey());
		} catch (javax.jdo.JDOObjectNotFoundException ex) {
			contains = false;
		} finally {
			mgr.close();
		}
		return contains;
	}

	private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
	}

}
