package com.example.moventaa.bean;

import com.example.moventaa.bean.PMF;

import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiNamespace;
import com.google.api.server.spi.response.CollectionResponse;
import com.google.appengine.api.datastore.Cursor;
import com.google.appengine.datanucleus.query.JDOCursorHelper;

import java.util.HashMap;
import java.util.List;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

@Api(name = "vacacionesendpoint", namespace = @ApiNamespace(ownerDomain = "example.com", ownerName = "example.com", packagePath = "moventaa.bean"))
public class VacacionesEndpoint {

	/**
	 * This method lists all the entities inserted in datastore.
	 * It uses HTTP GET method and paging support.
	 *
	 * @return A CollectionResponse class containing the list of all entities
	 * persisted and a cursor to the next page.
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "listVacaciones")
	public CollectionResponse<Vacaciones> listVacaciones(
			@Nullable @Named("cursor") String cursorString,
			@Nullable @Named("limit") Integer limit) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<Vacaciones> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Vacaciones.class);
			if (cursorString != null && cursorString != "") {
				cursor = Cursor.fromWebSafeString(cursorString);
				HashMap<String, Object> extensionMap = new HashMap<String, Object>();
				extensionMap.put(JDOCursorHelper.CURSOR_EXTENSION, cursor);
				query.setExtensions(extensionMap);
			}

			if (limit != null) {
				query.setRange(0, limit);
			}

			execute = (List<Vacaciones>) query.execute();
			cursor = JDOCursorHelper.getCursor(execute);
			if (cursor != null)
				cursorString = cursor.toWebSafeString();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Vacaciones obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Vacaciones> builder().setItems(execute)
				.setNextPageToken(cursorString).build();
	}

	/**
	 * This method gets the entity having primary key id. It uses HTTP GET method.
	 *
	 * @param id the primary key of the java bean.
	 * @return The entity with primary key id.
	 */
	@ApiMethod(name = "getVacaciones")
	public Vacaciones getVacaciones(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		Vacaciones vacaciones = null;
		try {
			vacaciones = mgr.getObjectById(Vacaciones.class, id);
		} finally {
			mgr.close();
		}
		return vacaciones;
	}

	/**
	 * This inserts a new entity into App Engine datastore. If the entity already
	 * exists in the datastore, an exception is thrown.
	 * It uses HTTP POST method.
	 *
	 * @param vacaciones the entity to be inserted.
	 * @return The inserted entity.
	 */
	@ApiMethod(name = "insertVacaciones")
	public Vacaciones insertVacaciones(Vacaciones vacaciones) {
		PersistenceManager mgr = getPersistenceManager();
		mgr.currentTransaction().begin();
		try {
		//	if (containsVacaciones(vacaciones)) {
		//		throw new EntityExistsException("Object already exists");
		//	}
			mgr.makePersistent(vacaciones);
		    mgr.currentTransaction().commit();
		} finally {
			if (mgr.currentTransaction().isActive()) {
		        mgr.currentTransaction().rollback();
		    }
			mgr.close();
		}
		return vacaciones;
	}

	/**
	 * This method is used for updating an existing entity. If the entity does not
	 * exist in the datastore, an exception is thrown.
	 * It uses HTTP PUT method.
	 *
	 * @param vacaciones the entity to be updated.
	 * @return The updated entity.
	 */
	@ApiMethod(name = "updateVacaciones")
	public Vacaciones updateVacaciones(Vacaciones vacaciones) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (!containsVacaciones(vacaciones)) {
				throw new EntityNotFoundException("Object does not exist");
			}
			mgr.makePersistent(vacaciones);
		} finally {
			mgr.close();
		}
		return vacaciones;
	}

	/**
	 * This method removes the entity with primary key id.
	 * It uses HTTP DELETE method.
	 *
	 * @param id the primary key of the entity to be deleted.
	 */
	@ApiMethod(name = "removeVacaciones")
	public void removeVacaciones(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			Vacaciones vacaciones = mgr.getObjectById(Vacaciones.class, id);
			mgr.deletePersistent(vacaciones);
		} finally {
			mgr.close();
		}
	}

	private boolean containsVacaciones(Vacaciones vacaciones) {
		PersistenceManager mgr = getPersistenceManager();
		boolean contains = true;
		try {
			mgr.getObjectById(Vacaciones.class, vacaciones.getKey());
		} catch (javax.jdo.JDOObjectNotFoundException ex) {
			contains = false;
		} finally {
			mgr.close();
		}
		return contains;
	}

	private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
	}

}
