package com.example.moventaa.endpoint;

import java.util.HashMap;
import java.util.List;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;

import com.example.moventaa.bean.Centro;
import com.example.moventaa.bean.PMF;
import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiNamespace;
import com.google.api.server.spi.response.CollectionResponse;
import com.google.appengine.api.datastore.Cursor;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;
import com.google.appengine.datanucleus.query.JDOCursorHelper;

@Api(name = "centroendpoint", namespace = @ApiNamespace(ownerDomain = "example.com", ownerName = "example.com", packagePath = "moventaa.bean"))
public class CentroEndpoint {

	
	
	/**
	 * Find centro by filter
	 * @param cursorString
	 * @param limit
	 * @return
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "findCentroFilter")
	public CollectionResponse<Centro> findCentroFilter(Centro centro) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<Centro> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Centro.class);
			
			HashMap<String, Object> params = new HashMap<String, Object>();
			
			if(centro.getNombreCentro()!=null){
				//query.setFilter("nombreCentro == nombre && localidadID == localidadID");
				query.setFilter("nombreCentro == nombre");
				query.declareParameters("String nombre");
				params.put( "nombre", centro.getNombreCentro() );
			}
			
			if(centro.getKey()!=null){
				query.setFilter("key == centroid");
				Key key = KeyFactory.createKey(Centro.class.getSimpleName(),centro.getKey().getName());
				//mgr.getObjectById(Centro.class, key);
				query.declareParameters("com.google.appengine.api.datastore.Key centroid");
				params.put("centroid", key);
				
			}
			
			if(centro.getLocalidadID()!=null){
				query.setFilter("localidadID == localidadID");
//				query.declareParameters("String localidadID");
				params.put( "localidadID", centro.getLocalidadID() );
			}
			
			//query.setFilter("placeID == place");
			//params.put( "place", centro.getPlaceId());
			
			
			//query.declareParameters(Key.class.getName() +  " id");
			query.setOrdering("nombreCentro asc");
			
			execute = (List<Centro>) query.executeWithMap(params);

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Centro obj : execute)
				;
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Centro> builder().setItems(execute).build();
	}
	
	
	
	/**
	 * This method lists all the entities inserted in datastore.
	 * It uses HTTP GET method and paging support.
	 *
	 * @return A CollectionResponse class containing the list of all entities
	 * persisted and a cursor to the next page.
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "listCentro")
	public CollectionResponse<Centro> listCentro(
			@Nullable @Named("cursor") String cursorString,
			@Nullable @Named("limit") Integer limit) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<Centro> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Centro.class);
			if (cursorString != null && cursorString != "") {
				cursor = Cursor.fromWebSafeString(cursorString);
				HashMap<String, Object> extensionMap = new HashMap<String, Object>();
				extensionMap.put(JDOCursorHelper.CURSOR_EXTENSION, cursor);
				query.setExtensions(extensionMap);
			}

			if (limit != null) {
				query.setRange(0, limit);
			}

			execute = (List<Centro>) query.execute();
			cursor = JDOCursorHelper.getCursor(execute);
			if (cursor != null)
				cursorString = cursor.toWebSafeString();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Centro obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Centro> builder().setItems(execute)
				.setNextPageToken(cursorString).build();
	}

	/**
	 * This method gets the entity having primary key id. It uses HTTP GET method.
	 *
	 * @param id the primary key of the java bean.
	 * @return The entity with primary key id.
	 */
	@ApiMethod(name = "getCentro")
	public Centro getCentro(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		Centro centro = null;
		try {
			centro = mgr.getObjectById(Centro.class, id);
		} finally {
			mgr.close();
		}
		return centro;
	}

	/**
	 * This inserts a new entity into App Engine datastore. If the entity already
	 * exists in the datastore, an exception is thrown.
	 * It uses HTTP POST method.
	 *
	 * @param centro the entity to be inserted.
	 * @return The inserted entity.
	 */
	@ApiMethod(name = "insertCentro")
	public Centro insertCentro(Centro centro) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (containsCentro(centro)) {
				throw new EntityExistsException("Object already exists");
			}
			mgr.makePersistent(centro);
		} finally {
			mgr.close();
		}
		return centro;
	}

	/**
	 * This method is used for updating an existing entity. If the entity does not
	 * exist in the datastore, an exception is thrown.
	 * It uses HTTP PUT method.
	 *
	 * @param centro the entity to be updated.
	 * @return The updated entity.
	 */
	@ApiMethod(name = "updateCentro")
	public Centro updateCentro(Centro centro) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (!containsCentro(centro)) {
				throw new EntityNotFoundException("Object does not exist");
			}
			mgr.makePersistent(centro);
		} finally {
			mgr.close();
		}
		return centro;
	}

	/**
	 * This method removes the entity with primary key id.
	 * It uses HTTP DELETE method.
	 *
	 * @param id the primary key of the entity to be deleted.
	 */
	@ApiMethod(name = "removeCentro")
	public void removeCentro(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			Centro centro = mgr.getObjectById(Centro.class, id);
			mgr.deletePersistent(centro);
		} finally {
			mgr.close();
		}
	}

	private boolean containsCentro(Centro centro) {
		PersistenceManager mgr = getPersistenceManager();
		boolean contains = true;
		try {
			mgr.getObjectById(Centro.class, centro.getKey());
		} catch (javax.jdo.JDOObjectNotFoundException ex) {
			contains = false;
		} finally {
			mgr.close();
		}
		return contains;
	}

	private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
	}

}
