package com.example.moventaa.endpoint;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import com.example.moventaa.bean.Centro;
import com.example.moventaa.bean.Empleado;
import com.example.moventaa.bean.Localidad;
import com.example.moventaa.bean.Nominas;
import com.example.moventaa.bean.PMF;
import com.example.moventaa.bean.Producto;
import com.example.moventaa.bean.ProductoInfo;
import com.example.moventaa.bean.ProductosCentro;
import com.example.moventaa.bean.Provincia;
import com.example.moventaa.bean.StockSolicitado;
import com.example.moventaa.bean.StockSolicitadoEndpoint;
import com.example.moventaa.bean.Vacaciones;
import com.example.moventaa.bean.VacacionesEndpoint;
import com.example.moventaa.bean.VentaProductoCentro;
import com.example.moventaa.placeutil.PlaceInfo;
import com.example.moventaa.placeutil.PlacesHelper;
import com.google.api.server.spi.ServiceException;
import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiNamespace;
import com.google.api.server.spi.response.BadRequestException;
import com.google.api.server.spi.response.CollectionResponse;
import com.google.appengine.api.datastore.Cursor;
import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.DatastoreServiceFactory;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.FetchOptions;
import com.google.appengine.api.datastore.GeoPt;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;
import com.google.appengine.api.datastore.PreparedQuery;
import com.google.appengine.api.datastore.Query.CompositeFilterOperator;
import com.google.appengine.api.datastore.Query.Filter;
import com.google.appengine.api.datastore.Query.FilterOperator;
import com.google.appengine.api.datastore.Query.FilterPredicate;
import com.google.appengine.api.datastore.QueryResultIterable;
import com.google.appengine.api.users.User;
import com.google.appengine.datanucleus.JDODatastoreBridge;
import com.google.appengine.datanucleus.query.JDOCursorHelper;


@Api(name = "moventaaendpoint", namespace = @ApiNamespace(ownerDomain = "example.com", ownerName = "example.com", packagePath = "moventaa.bean"))
public class MoventaaEndpoint {

	
	/**
	 * Metodo para la busqueda de un producto por
	 * nombre de producto
	 * 
	 * @param cursorString
	 * @param limit
	 * @return
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "findProductoByName")
	public CollectionResponse<Producto> findProductoByName(
			@Nullable @Named("name") String nombrefind) {

		List<Producto> results = new LinkedList<Producto>(); 
		PersistenceManager mgr = null;
		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Producto.class);
			List<Producto> execute = (List<Producto>) query.execute();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Producto obj : execute){
				if(obj.getNombre().toUpperCase().contains(nombrefind.toUpperCase())){
					results.add(obj);
				}
			}
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Producto> builder().setItems(results).build();
	}
	
	private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
	}
	
	
	/**
	 * Busca un empleado por su identificador ID
	 * @param id
	 * @return
	 */
	@ApiMethod ( name = "searchEmpleadoById" )
	public Empleado searchEmpleadoById(@Named ( "id" ) String id){
		PersistenceManager mgr = getPersistenceManager();
		
		DatastoreService datastore = DatastoreServiceFactory.getDatastoreService();
		List<Empleado> execute = null;
		Empleado empleado = null;
        boolean contains = true;
        
        try {
        	Key k = KeyFactory.createKey(Empleado.class.getSimpleName(), id);
    		try {
    			empleado = mgr.getObjectById(Empleado.class, k);
    		} finally {
    			mgr.close();
    		}
    		
    		
        	
        } catch (javax.jdo.JDOObjectNotFoundException ex) {
            contains = false;
	    } finally {
	            mgr.close();
	    }
	
	    return empleado;
    }		
			
	
	
	/**
	 * Este metodo busca los datos del empleado en base a un filtro de tipo Empleado
	 * @param queryString
	 * @return
	 */
	@ApiMethod ( name = "search.empleado" )
	public  CollectionResponse<Empleado> searchEmpleado (
			            Empleado empleado)   {
//						@Named ( "empleado" ) Empleado empleado)   {
		
		PersistenceManager mgr = getPersistenceManager();
		List<Empleado> execute = new ArrayList<Empleado>();
		
		DatastoreService datastore = DatastoreServiceFactory.getDatastoreService();
		
        boolean contains = true;
        try {
        	List<Filter> filterCollect = new ArrayList<Filter>();
        	filterCollect.add(new FilterPredicate("nombre", FilterOperator.EQUAL, empleado.getNombre()));
        	filterCollect.add(new FilterPredicate("primerApellido", FilterOperator.EQUAL, empleado.getPrimerApellido()));
        	filterCollect.add(new FilterPredicate("segundoApellido", FilterOperator.EQUAL, empleado.getSegundoApellido()));
        	filterCollect.add(new FilterPredicate("docIdentificacion", FilterOperator.EQUAL, empleado.getDocIdentificacion()));
        	//filterCollect.add(new FilterPredicate("fechaNacimiento", FilterOperator.EQUAL, empleado.getFechaNacimiento()));
        	
        	Filter filtercollect =
        			  CompositeFilterOperator.and(filterCollect);
        	com.google.appengine.api.datastore.Query query = new com.google.appengine.api.datastore.Query("Empleado").setFilter(filtercollect);
        	
        	PreparedQuery prepared = datastore.prepare(query);
        	if(prepared.countEntities(FetchOptions.Builder.withDefaults())==1){
	        	QueryResultIterable<Entity> queryResultIterable = prepared.asQueryResultIterable();
	        	JDODatastoreBridge bridge = new JDODatastoreBridge();
	        	execute = bridge.toJDOResult(mgr, Empleado.class, queryResultIterable);
        	}
      	  
        } catch (javax.jdo.JDOObjectNotFoundException ex) {
                contains = false;
        } finally {
                mgr.close();
        }
	
	    return CollectionResponse.<Empleado> builder().setItems(execute).build();
	  }
	
	/**
	 * Inserta las vacaciones para el empleado
	 * @param idEmpleado
	 * @param vacaciones
	 * @return
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod ( name = "insertVacacionesEmpleado" )
	public void insertVacacionesEmpleado (
							@Nullable @Named("idEmpleado") String idEmpleado,
							@Nullable @Named("vacaciones") Long vacaciones){
		if(idEmpleado!=null){
			VacacionesEndpoint vacendpoint = new VacacionesEndpoint();
			
			//Eliminamos antes las vacaciones que pudieran haber en el mes
			Calendar calendar = Calendar.getInstance();
			calendar.setTimeInMillis(vacaciones);
			
			CollectionResponse<Vacaciones> lstvac = getVacacionesEmpleado(idEmpleado, calendar.get(Calendar.MONTH)+1, calendar.get(Calendar.YEAR));
			Collection<Vacaciones> items = lstvac.getItems();
			for (Vacaciones vacaciones2 : items) {
				vacendpoint.removeVacaciones(vacaciones2.getKey().getId());
			}
			
			Vacaciones vac = new Vacaciones();
			vac.setEmpleadoID(idEmpleado);
			vac.setDate(new Date(vacaciones));
			Vacaciones insertvac = vacendpoint.insertVacaciones(vac);
		}	
	}
	
	
	/**
	 * Obtiene las vacaciones de un empleado para un 
	 * annio y mes dado
	 * @param idEmpleado
     */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod ( name = "getVacacionesEmpleado" )
	public CollectionResponse<Vacaciones> getVacacionesEmpleado(
			       @Nullable @Named("idEmpleado") String idEmpleado,
			       @Nullable @Named("mes") Integer mes,
			       @Nullable @Named("annio") Integer annio){
		
		PersistenceManager mgr = getPersistenceManager();
		List<Vacaciones> execute;
		try {
			
			Calendar calendar = Calendar.getInstance();
			//calendar.add(Calendar.DAY_OF_MONTH, -1);
			calendar.set(Calendar.DATE, 01);
			calendar.set(Calendar.MONTH, mes-1);
			calendar.set(Calendar.YEAR, annio);
			Date monthStart = new Date(calendar.getTimeInMillis());
			
			
			calendar.add(Calendar.MONTH, 1);
			Date mothEnd = new Date(calendar.getTimeInMillis()); 
			
			//Obtenemos las ventas realizadas en el dia por el empleado
			Query query = mgr.newQuery(Vacaciones.class,
			        "empleadoID == idempleado " +
					"&& this.date >= startDate && this.date <= endDate");
			query.declareParameters("String idempleado, java.util.Date startDate, java.util.Date endDate");
            execute =  (List<Vacaciones>)query.execute(idEmpleado,monthStart,mothEnd);
	//		datos.setEmpleadoID(idEmpleado);
			//datos.setCentro(idEmpleado);
			
		} finally {
			mgr.close();
		}
		return CollectionResponse.<Vacaciones> builder().setItems(execute).build();
	}
	
		/**
	 * Incluye una solicitud de stock a la base de datos
	 * @param idEmpleado
	 * @param idproducto
	 * @param idcentro
	 * @param cantidad
	 */
	@ApiMethod ( name = "anadirSolicitudStock" )
	public void anadirSolicitudStock (
			@Nullable @Named("idEmpleado") String idEmpleado,
			@Nullable @Named("idProducto") String idproducto,
			@Nullable @Named("idCentro") String idcentro,
			@Nullable @Named("cantidad") String cantidad){

		
		StockSolicitadoEndpoint endpoint = new StockSolicitadoEndpoint();
		StockSolicitado stocksolicitado = new StockSolicitado();
		stocksolicitado.setCantidad(Integer.parseInt(cantidad));
		stocksolicitado.setEmpleadoID(idEmpleado);
		stocksolicitado.setCentroID(idcentro);
		stocksolicitado.setProductoID(idproducto);
		stocksolicitado.setFechaSolicitud(new Date());
		endpoint.insertStockSolicitado(stocksolicitado);
   }

	
	/**
	 * Busca un producto por su id
	 * @param idProducto
	 * @param idCentro
	 * @return
	 */
	@ApiMethod(name = "findInfoProductoById")
	public ProductoInfo findInfoProductoById(
			   @Named("id") String idProducto,
			   @Named("idCentro") String idCentro) {
		
		ProductoInfo productoinfo = new ProductoInfo();
		
		ProductoEndpoint producto = new ProductoEndpoint();
		productoinfo.setProducto(producto.findProductoById(idProducto));
		
		ProductosCentroEndpoint prodcentro = new ProductosCentroEndpoint();
		CollectionResponse<ProductosCentro> lst = prodcentro.findProductosByCentro(idCentro);
		for (ProductosCentro it : lst.getItems()) {
			if(it.getProductoId().equals(idProducto)){
				productoinfo.setStock(it.getStock());	
			}
		}
		
		CollectionResponse<StockSolicitado> lstcol = findSolicitudStockByCentro(idCentro);
		for (StockSolicitado it : lstcol.getItems()) {
			if(it.getProductoID().equals(idProducto)){
				productoinfo.setSolicitado(Integer.toString(it.getCantidad()));
			}
		}
		return productoinfo;
	}
	
	/**
	 * Busca un producto por su barcode
	 * @param barcode
	 * @param idCentro
	 * @return
	 */
	@ApiMethod(name = "findInfoProductoByBarcode")
	public ProductoInfo findInfoProductoByBarcode(
			   @Named("barcode") String barcode,
			   @Named("idCentro") String idCentro) {
		
		ProductoEndpoint productoendpoint = new ProductoEndpoint();
		Producto producto = productoendpoint.findProductoByBarCode(barcode);
		
		if(producto!=null){
		   return findInfoProductoById(producto.getKey().getName(), idCentro);
		}
		return null;
	}
	
	/**
	 * Encuentra las solicitudes realizadas de stock por centro 
	 * y producto
	 * 
	 * @param idCentro
	 * @return
	 */
	@ApiMethod(name = "findSolicitudStockByCentro")
	public CollectionResponse<StockSolicitado> findSolicitudStockByCentro(
			@Nullable @Named("idCentro") String idCentro) {

		PersistenceManager mgr = null;
		List<StockSolicitado> execute = null;
		
		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(StockSolicitado.class);
			query.setFilter("centroID == centroid");
			query.declareParameters("String centroid");
			
			execute = (List<StockSolicitado>) query.execute(idCentro);

		} finally {
			mgr.close();
		}

		return CollectionResponse.<StockSolicitado> builder().setItems(execute).build();
	}
	
	
	
	  /**
	   * This method lists nearby places. It uses HTTP GET method.
	   *
	   * @return List of nearby places.
	   */
	@ApiMethod(name = "findCentroByCoords")
	public List<Centro> findCentroByCoords(@Named("longitude") String longitudeString,
	      @Named("latitude") String latitudeString, @Named("distanceInKm") long distanceInKm,
	      @Named("count") int count, User user) throws ServiceException {

	    float latitude;
	    float longitude;
	    GeoPt location;

	    try {
	      latitude = (float) Double.parseDouble(latitudeString);
	    } catch (Exception e) {
	      throw new BadRequestException("Invalid value of 'latitude' argument");
	    }

	    try {
	      longitude = (float) Double.parseDouble(longitudeString);
	    } catch (Exception e) {
	      throw new BadRequestException("Invalid value of 'longitude' argument");
	    }

	    try {
	      location = new GeoPt(latitude, longitude);
	    } catch (Exception e) {
	      throw new BadRequestException("Invalid pair of 'latitude' and 'longitude' arguments");
	    }

	    // limit the result set to up to 100 places within up to 200 km
	    if (count > 100) {
	      count = 100;
	    } else if (count <= 0) {
	      throw new BadRequestException("Invalid value of 'count' argument");
	    }

	    if (distanceInKm > 200) {
	      distanceInKm = 200;
	    } else if (distanceInKm < 0) {
	      throw new BadRequestException("Invalid value of 'distanceInKm' argument");
	    }

	    List<PlaceInfo> places = PlacesHelper.getPlaces(location, 1000 * distanceInKm, count);
	    List<Centro> centros = new ArrayList<Centro>(places);
	    
	    return centros;
	  }
	
	/**
	 * This method lists all the entities inserted in datastore.
	 * It uses HTTP GET method and paging support.
	 *
	 * @return A CollectionResponse class containing the list of all entities
	 * persisted and a cursor to the next page.
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "listProvincia")
	public CollectionResponse<Provincia> listProvincia(
			@Nullable @Named("cursor") String cursorString,
			@Nullable @Named("limit") Integer limit) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<Provincia> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Provincia.class);
			query.setOrdering("provincia asc");
			if (cursorString != null && cursorString != "") {
				cursor = Cursor.fromWebSafeString(cursorString);
				HashMap<String, Object> extensionMap = new HashMap<String, Object>();
				extensionMap.put(JDOCursorHelper.CURSOR_EXTENSION, cursor);
				query.setExtensions(extensionMap);
			}

			if (limit != null) {
				query.setRange(0, limit);
			}

			execute = (List<Provincia>) query.execute();
			cursor = JDOCursorHelper.getCursor(execute);
			if (cursor != null)
				cursorString = cursor.toWebSafeString();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Provincia obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Provincia> builder().setItems(execute)
				.setNextPageToken(cursorString).build();
	}
	
	
	/**
	 * Listado de localidades por provincia
	 * @param cursorString
	 * @return
	 */
	@ApiMethod(name = "localidadesByProvicia")
	public CollectionResponse<Localidad> localidadesByProvicia(
			@Nullable @Named("idProvincia") String idprovincia) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<Localidad> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Localidad.class);
			query.setFilter("provinciaID == id");
			query.setOrdering("localidad asc");
			//query.declareParameters(Key.class.getName() +  " id");
			query.declareParameters("String id");
			
			execute = (List<Localidad>) query.execute(idprovincia);
			cursor = JDOCursorHelper.getCursor(execute);
			

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Localidad obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Localidad> builder().setItems(execute).build();
	}

	
	/**
	 * Lista las nominas del empleado
	 * 
	 * @param idempleado
	 * @return
	 */
	@ApiMethod(name = "listNominasEmpleado")
	public CollectionResponse<Nominas> listNominasEmpleado(
			     @Named("idEmpleado") String idempleado){
		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<Nominas> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(Nominas.class);
			query.setFilter("empleadoID == id");
			query.setOrdering("date asc");
			//query.declareParameters(Key.class.getName() +  " id");
			query.declareParameters("String id");
			
			execute = (List<Nominas>) query.execute(idempleado);
			cursor = JDOCursorHelper.getCursor(execute);
			

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (Nominas obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<Nominas> builder().setItems(execute).build();
	}
	
}
