package com.example.moventaa.endpoint;

import java.util.HashMap;
import java.util.List;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;

import com.example.moventaa.bean.PMF;
import com.example.moventaa.bean.ProductosCentro;
import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiNamespace;
import com.google.api.server.spi.response.CollectionResponse;
import com.google.appengine.api.datastore.Cursor;
import com.google.appengine.datanucleus.query.JDOCursorHelper;

@Api(name = "productoscentroendpoint", namespace = @ApiNamespace(ownerDomain = "example.com", ownerName = "example.com", packagePath = "moventaa.bean"))
public class ProductosCentroEndpoint {

	
	/**
	 * Metodo que encuentra los productos asociados a un centro
	 * @param idCentro
	 * @return
	 */
	@ApiMethod(name = "findProductosByCentro")
	public CollectionResponse<ProductosCentro> findProductosByCentro(
			@Nullable @Named("idCentro") String idCentro) {

		PersistenceManager mgr = null;
		List<ProductosCentro> execute = null;
		
		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(ProductosCentro.class);
			query.setFilter("centroId == centroid");
			query.declareParameters("String centroid");
			
			execute = (List<ProductosCentro>) query.execute(idCentro);

		} finally {
			mgr.close();
		}

		return CollectionResponse.<ProductosCentro> builder().setItems(execute).build();
	}
	
	/**
	 * Metodo que encuentra por id producto y id centro
	 * @param id
	 * @return
	 */
	@ApiMethod(name = "updateStockProductoCentro")
	public ProductosCentro updateStockProductoCentro(@Named("centroid") String centroid,
												     @Named("productoid") String productoid,
												     @Named("stock") Integer stock) {
		PersistenceManager mgr = getPersistenceManager();
		ProductosCentro productoscentro = null;
		
		mgr = getPersistenceManager();
		Query query = mgr.newQuery(ProductosCentro.class);
		query.setFilter("centroId == centroid && productoId == productoid");
		query.declareParameters("String centroid,String productoid");
		
		try {
			List<ProductosCentro> execute = (List<ProductosCentro>) query.execute(centroid,productoid);
			productoscentro = execute.get(0);
			
			
			int stockval = Integer.parseInt(productoscentro.getStock());
			productoscentro.setStock(Integer.toString(stockval - stock));
			updateProductosCentro(productoscentro);
		} finally {
			mgr.close();
		}
		return productoscentro;
	}

	
	
	

	/**
	 * This method lists all the entities inserted in datastore.
	 * It uses HTTP GET method and paging support.
	 *
	 * @return A CollectionResponse class containing the list of all entities
	 * persisted and a cursor to the next page.
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "listProductosCentro")
	public CollectionResponse<ProductosCentro> listProductosCentro(
			@Nullable @Named("cursor") String cursorString,
			@Nullable @Named("limit") Integer limit) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<ProductosCentro> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(ProductosCentro.class);
			if (cursorString != null && cursorString != "") {
				cursor = Cursor.fromWebSafeString(cursorString);
				HashMap<String, Object> extensionMap = new HashMap<String, Object>();
				extensionMap.put(JDOCursorHelper.CURSOR_EXTENSION, cursor);
				query.setExtensions(extensionMap);
			}

			if (limit != null) {
				query.setRange(0, limit);
			}

			execute = (List<ProductosCentro>) query.execute();
			cursor = JDOCursorHelper.getCursor(execute);
			if (cursor != null)
				cursorString = cursor.toWebSafeString();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (ProductosCentro obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<ProductosCentro> builder().setItems(execute)
				.setNextPageToken(cursorString).build();
	}

	/**
	 * This method gets the entity having primary key id. It uses HTTP GET method.
	 *
	 * @param id the primary key of the java bean.
	 * @return The entity with primary key id.
	 */
	@ApiMethod(name = "getProductosCentro")
	public ProductosCentro getProductosCentro(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		ProductosCentro productoscentro = null;
		try {
			productoscentro = mgr.getObjectById(ProductosCentro.class, id);
		} finally {
			mgr.close();
		}
		return productoscentro;
	}

	/**
	 * This inserts a new entity into App Engine datastore. If the entity already
	 * exists in the datastore, an exception is thrown.
	 * It uses HTTP POST method.
	 *
	 * @param productoscentro the entity to be inserted.
	 * @return The inserted entity.
	 */
	@ApiMethod(name = "insertProductosCentro")
	public ProductosCentro insertProductosCentro(ProductosCentro productoscentro) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (containsProductosCentro(productoscentro)) {
				throw new EntityExistsException("Object already exists");
			}
			mgr.makePersistent(productoscentro);
		} finally {
			mgr.close();
		}
		return productoscentro;
	}

	/**
	 * This method is used for updating an existing entity. If the entity does not
	 * exist in the datastore, an exception is thrown.
	 * It uses HTTP PUT method.
	 *
	 * @param productoscentro the entity to be updated.
	 * @return The updated entity.
	 */
	@ApiMethod(name = "updateProductosCentro")
	public ProductosCentro updateProductosCentro(ProductosCentro productoscentro) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (!containsProductosCentro(productoscentro)) {
				throw new EntityNotFoundException("Object does not exist");
			}
			mgr.makePersistent(productoscentro);
		} finally {
			mgr.close();
		}
		return productoscentro;
	}

	/**
	 * This method removes the entity with primary key id.
	 * It uses HTTP DELETE method.
	 *
	 * @param id the primary key of the entity to be deleted.
	 */
	@ApiMethod(name = "removeProductosCentro")
	public void removeProductosCentro(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			ProductosCentro productoscentro = mgr.getObjectById(
					ProductosCentro.class, id);
			mgr.deletePersistent(productoscentro);
		} finally {
			mgr.close();
		}
	}

	private boolean containsProductosCentro(ProductosCentro productoscentro) {
		PersistenceManager mgr = getPersistenceManager();
		boolean contains = true;
		try {
			mgr.getObjectById(ProductosCentro.class, productoscentro.getKey());
		} catch (javax.jdo.JDOObjectNotFoundException ex) {
			contains = false;
		} finally {
			mgr.close();
		}
		return contains;
	}

	private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
	}

}
