package com.example.moventaa.endpoint;

import com.example.moventaa.bean.EmpleadoProductoVentas;
import com.example.moventaa.bean.PMF;
import com.example.moventaa.bean.Producto;
import com.example.moventaa.bean.ProductosCentro;
import com.example.moventaa.bean.VentaProductoCentro;
import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiNamespace;
import com.google.api.server.spi.response.CollectionResponse;
import com.google.appengine.api.datastore.Cursor;
import com.google.appengine.api.datastore.KeyFactory;
import com.google.appengine.datanucleus.query.JDOCursorHelper;

import java.sql.Date;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

@Api(name = "ventaproductocentroendpoint", namespace = @ApiNamespace(ownerDomain = "example.com", ownerName = "example.com", packagePath = "moventaa.bean"))
public class VentaProductoCentroEndpoint {

	/**
	 * This method lists all the entities inserted in datastore.
	 * It uses HTTP GET method and paging support.
	 *
	 * @return A CollectionResponse class containing the list of all entities
	 * persisted and a cursor to the next page.
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	@ApiMethod(name = "listVentaProductoCentro")
	public CollectionResponse<VentaProductoCentro> listVentaProductoCentro(
			@Nullable @Named("cursor") String cursorString,
			@Nullable @Named("limit") Integer limit) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		List<VentaProductoCentro> execute = null;

		try {
			mgr = getPersistenceManager();
			Query query = mgr.newQuery(VentaProductoCentro.class);
			if (cursorString != null && cursorString != "") {
				cursor = Cursor.fromWebSafeString(cursorString);
				HashMap<String, Object> extensionMap = new HashMap<String, Object>();
				extensionMap.put(JDOCursorHelper.CURSOR_EXTENSION, cursor);
				query.setExtensions(extensionMap);
			}

			if (limit != null) {
				query.setRange(0, limit);
			}

			execute = (List<VentaProductoCentro>) query.execute();
			cursor = JDOCursorHelper.getCursor(execute);
			if (cursor != null)
				cursorString = cursor.toWebSafeString();

			// Tight loop for fetching all entities from datastore and accomodate
			// for lazy fetch.
			for (VentaProductoCentro obj : execute)
				;
		} finally {
			mgr.close();
		}

		return CollectionResponse.<VentaProductoCentro> builder()
				.setItems(execute).setNextPageToken(cursorString).build();
	}
	

	
	@ApiMethod(name = "getEmpleadoVentasProductoCentro")
	public EmpleadoProductoVentas getEmpleadoVentasProductoCentro(
			@Nullable @Named("idCentro") String idCentro,
			@Nullable @Named("idEmpleado") String idEmpleado) {

		PersistenceManager mgr = null;
		Cursor cursor = null;
		EmpleadoProductoVentas datos = new EmpleadoProductoVentas();

		try {
			mgr = getPersistenceManager();
			
			
			Calendar calendar = Calendar.getInstance();
			calendar.add(Calendar.DAY_OF_MONTH, -1);
			java.util.Date today = new java.util.Date(calendar.getTimeInMillis());
			//Obtenemos las ventas realizadas en el dia por el empleado
			Query query = mgr.newQuery(VentaProductoCentro.class,
			        "centroId == idcentro && empleadoID == idempleado " +
					//"&& this.fechaVenta >= startDate && this.fechaVenta <= endDat"
					"&& this.fechaVenta >= startDate"
                    //"PARAMETERS java.util.Date startDate, java.util.Date endDate"
                    );
			query.declareParameters("String idcentro, String idempleado, java.util.Date startDate");
			List<VentaProductoCentro> ventasexec = (List<VentaProductoCentro>) query.execute(idCentro,idEmpleado,today);
			datos.setEmpleadoID(idEmpleado);
			//datos.setCentro(idEmpleado);
			
			//Cargamos los datos de las ventas
			for (VentaProductoCentro venta : ventasexec) {
			 	Integer contador = datos.getProductosVendidos().get(venta.getProductoId());
				if(contador == null){
					contador = 0;
				}
				datos.getProductosVendidos().put(venta.getProductoId(), ++contador);
			}
			
			//Recuperamos todos los productos que se pueden vender en el centro
			ProductosCentroEndpoint productoscentro = new ProductosCentroEndpoint();
			CollectionResponse<ProductosCentro> resutlado = productoscentro.findProductosByCentro(idCentro);
			
			List<Producto> returncollection = new ArrayList<Producto>();
			ProductoEndpoint prodendpoint = new ProductoEndpoint(); 
			for (ProductosCentro obj : resutlado.getItems()){
				Producto producto = prodendpoint.findProductoById(obj.getProductoId());
				returncollection.add(producto);
				//Cargamos los datos de stock
				datos.getProductoStock().put(obj.getProductoId(), Integer.parseInt(obj.getStock()));
			}
			datos.setProductos(returncollection);
			
		} finally {
			mgr.close();
		}

		return datos;
	}

	
	/**
	 * This method gets the entity having primary key id. It uses HTTP GET method.
	 *
	 * @param id the primary key of the java bean.
	 * @return The entity with primary key id.
	 */
	@ApiMethod(name = "getVentaProductoCentro")
	public VentaProductoCentro getVentaProductoCentro(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		VentaProductoCentro ventaproductocentro = null;
		try {
			ventaproductocentro = mgr.getObjectById(VentaProductoCentro.class,
					id);
		} finally {
			mgr.close();
		}
		return ventaproductocentro;
	}

	/**
	 * This inserts a new entity into App Engine datastore. If the entity already
	 * exists in the datastore, an exception is thrown.
	 * It uses HTTP POST method.
	 *
	 * @param ventaproductocentro the entity to be inserted.
	 * @return The inserted entity.
	 */
	@ApiMethod(name = "insertVentaProductoCentro")
	public VentaProductoCentro insertVentaProductoCentro(
			VentaProductoCentro ventaproductocentro) {
		PersistenceManager mgr = getPersistenceManager();
		
		if(ventaproductocentro.getFechaVenta()==null){
			ventaproductocentro.setFechaVenta(new java.util.Date(GregorianCalendar.getInstance().getTimeInMillis()));
		}
		try {
			//if (containsVentaProductoCentro(ventaproductocentro)) {
			//	throw new EntityExistsException("Object already exists");
			//}
			mgr.makePersistent(ventaproductocentro);
			
			//Eliminamos tantas entradas de producto stock
			ProductosCentroEndpoint productos = new ProductosCentroEndpoint();
			productos.updateStockProductoCentro(ventaproductocentro.getCentroId(), 
						                        ventaproductocentro.getProductoId(),
						                        1);
			
		} finally {
			mgr.close();
		}
		return ventaproductocentro;
	}

	/**
	 * This method is used for updating an existing entity. If the entity does not
	 * exist in the datastore, an exception is thrown.
	 * It uses HTTP PUT method.
	 *
	 * @param ventaproductocentro the entity to be updated.
	 * @return The updated entity.
	 */
	@ApiMethod(name = "updateVentaProductoCentro")
	public VentaProductoCentro updateVentaProductoCentro(
			VentaProductoCentro ventaproductocentro) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			if (!containsVentaProductoCentro(ventaproductocentro)) {
				throw new EntityNotFoundException("Object does not exist");
			}
			mgr.makePersistent(ventaproductocentro);
		} finally {
			mgr.close();
		}
		return ventaproductocentro;
	}

	/**
	 * This method removes the entity with primary key id.
	 * It uses HTTP DELETE method.
	 *
	 * @param id the primary key of the entity to be deleted.
	 */
	@ApiMethod(name = "removeVentaProductoCentro")
	public void removeVentaProductoCentro(@Named("id") Long id) {
		PersistenceManager mgr = getPersistenceManager();
		try {
			VentaProductoCentro ventaproductocentro = mgr.getObjectById(
					VentaProductoCentro.class, id);
			mgr.deletePersistent(ventaproductocentro);
		} finally {
			mgr.close();
		}
	}

	private boolean containsVentaProductoCentro(
			VentaProductoCentro ventaproductocentro) {
		PersistenceManager mgr = getPersistenceManager();
		boolean contains = true;
		try {
			mgr.getObjectById(VentaProductoCentro.class,
					ventaproductocentro.getKey());
		} catch (javax.jdo.JDOObjectNotFoundException ex) {
			contains = false;
		} finally {
			mgr.close();
		}
		return contains;
	}

	private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
	}

}
