package com.example.moventaa.placeutil;

import java.io.IOException;
import java.util.List;

import javax.jdo.PersistenceManager;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.example.moventaa.EMF;
import com.example.moventaa.bean.Centro;
import com.example.moventaa.bean.PMF;
import com.google.appengine.api.datastore.GeoPt;
import com.google.appengine.api.search.Document;
import com.google.appengine.api.search.GetRequest;
import com.google.appengine.api.search.GetResponse;
import com.google.appengine.api.search.Index;
import com.google.appengine.api.search.PutException;
import com.google.appengine.api.search.StatusCode;

/**
 * HttpServlet for handling maintenance tasks.
 */
public class MaintenanceTasksServlet extends HttpServlet {

  private static final long serialVersionUID = 1L;

  @Override
  public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
    resp.setContentType("text/plain");
    if (!buildSearchIndexForPlaces()) {
      resp.getWriter().println("MaintenanceTasks failed. Try again by refreshing the page.");
      return;
    }
    resp.getWriter().println("MaintenanceTasks completed");
  }

  @SuppressWarnings({"cast", "unchecked"})
  private boolean buildSearchIndexForPlaces() {
    Index index = PlacesHelper.getIndex();

    removeAllDocumentsFromIndex();

    try {
      PersistenceManager mgr = getPersistenceManager();
      //Query query = mgr.createQuery("select from Centro as Centro");
      
      javax.jdo.Query query = mgr.newQuery(Centro.class);
      for (Object obj : (List<Object>) query.execute()) {
    	Centro place = (Centro) obj;
    	
    	
    	String[] data = place.getLocation().split(",");
    	GeoPt geopt = new GeoPt(Float.parseFloat(data[0]),
    			                Float.parseFloat(data[1]));
    	
    	
        Document placeAsDocument = PlacesHelper.buildDocument(
        	place.getKey().getName(),	
            place.getPlaceId(), place.getNombreCentro(), place.getDireccion(), geopt);
        try {
          index.put(placeAsDocument);
        } catch (PutException e) {
          if (StatusCode.TRANSIENT_ERROR.equals(e.getOperationResult().getCode())) {
            return false;
          }
        }
      }
    } catch (Exception e) {
      return false;
    } finally {
      //mgr.close();
    }

    return true;
  }

  private void removeAllDocumentsFromIndex() {
    Index index = PlacesHelper.getIndex();

    GetRequest request = GetRequest.newBuilder().setReturningIdsOnly(true).build();

    GetResponse<Document> response = index.getRange(request);
    for (Document document : response.getResults()) {
      index.delete(document.getId());
    }
  }

  private static EntityManager getEntityManager() {
    return EMF.get().createEntityManager();
  }
  
  private static PersistenceManager getPersistenceManager() {
		return PMF.get().getPersistenceManager();
  }

}
