package com.example.moventaa;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Base64;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.Toast;
import butterknife.ButterKnife;
import butterknife.InjectView;
import butterknife.OnClick;

import com.example.moventaa.bean.bajaendpoint.Bajaendpoint;
import com.example.moventaa.bean.bajaendpoint.model.Baja;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.DateTime;

/**
 * Activity para implmentar la baja solicitad por un empleado
 * @author mmartinezced
 *
 */
public class BajaActivity extends Activity {

	static final int REQUEST_IMAGE_CAPTURE = 1;
	
	@InjectView(R.id.btnFoto) Button btnFoto;
	@InjectView(R.id.imageView1) ImageView mImageView;
	@InjectView(R.id.spinner_tiposbajas) Spinner tiposbajas;
	
	private File mFile;
	private Bitmap bm; 
	private ProgressDialog progressDialog;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_bajas);
		ButterKnife.inject(this);
	}
	
	/*
	 * Activa la camara
	 */
	@OnClick(R.id.btnFoto)
	public void onClickFoto(View view){
		File dir = new File(Environment.getExternalStorageDirectory() + "/dcim/Moventaa");
		try {
			if (!dir.exists()) {
	            dir.mkdirs();
	        }
			mFile = File.createTempFile("myImage", ".png", dir);
			Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
			takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(mFile));
			if (takePictureIntent.resolveActivity(getPackageManager()) != null) {
			        startActivityForResult(takePictureIntent, REQUEST_IMAGE_CAPTURE);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		 
	}
	
	/**
	 * Envia la baja al servidor
	 * @param view
	 */
	@OnClick(R.id.btnAtras)
	public void onClickStock(View view){
		
		if( mFile !=null ){
			progressDialog = ProgressDialog.show(this,
					   "", 
	                  "Enviando baja. Por favor, espere...",
	                  true);
		
			
			Baja content = new Baja();
			content.setDate(new DateTime(new java.util.Date()));
			final AppGlobal globalVariable = (AppGlobal) getApplicationContext();
			content.setIdEmpleado(globalVariable.getEmpleadoID());
			
	        ByteArrayOutputStream stream = new ByteArrayOutputStream();
            bm.compress(Bitmap.CompressFormat.PNG, 100, stream);
            byte[] byteArray = stream.toByteArray();
            content.setTipo(tiposbajas.getSelectedItem().toString());
			String stringEncodedImage = Base64.encodeToString(byteArray, Base64.DEFAULT);
			content.setImage(stringEncodedImage);
			
			
			BajaAsyncRetriever bajatask = new BajaAsyncRetriever();
			bajatask.execute(content);
		}else{
			Toast.makeText(getBaseContext(),
					"Primero haga una foto",
					Toast.LENGTH_LONG).show();
		}
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
	    if (requestCode == REQUEST_IMAGE_CAPTURE && resultCode == RESULT_OK) {
            try {
                bm = BitmapFactory.decodeFile(mFile.getAbsolutePath());
                mImageView.setImageBitmap(bm);
            } catch (Exception e) {
                Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
            }
	    }
	}
	
	
	/**
	 * Manda la foto al servidor
	 *
	 */
	private class BajaAsyncRetriever extends
			AsyncTask<Baja, Void, Baja> {

		@Override
		protected Baja doInBackground(Baja... params) {

			Bajaendpoint.Builder endpointBuilder = new Bajaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);

			endpointBuilder = CloudEndpointUtils.updateBuilder(endpointBuilder);

			Baja result = null;
			Bajaendpoint endpoint = endpointBuilder.build();

			try {
				result = endpoint.insertBaja(params[0]).execute();
				// result = endpoint.listEmpleado()

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				result = null;
			}
			return result;
		}

		@Override
		@SuppressWarnings("null")
		protected void onPostExecute(Baja result) {

			super.onPostExecute(result);
			progressDialog.dismiss();
			if (result == null) {
				Toast.makeText(getBaseContext(),
						"Error al enviar la baja",
						Toast.LENGTH_LONG).show();

			} else {
				Toast.makeText(getBaseContext(),
						"Baja enviada",
						Toast.LENGTH_LONG).show();
			}
		}
	}
	
	/*
	private void galleryAddPic() {
	    Intent mediaScanIntent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
	    File f = new File(mCurrentPhotoPath);
	    Uri contentUri = Uri.fromFile(f);
	    mediaScanIntent.setData(contentUri);
	    this.sendBroadcast(mediaScanIntent);
	}
	*/
}
