package com.example.moventaa;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.Toast;
import butterknife.ButterKnife;
import butterknife.InjectView;
import butterknife.OnClick;
import butterknife.OnItemSelected;

import com.example.moventaa.bean.moventaaendpoint.model.Localidad;
import com.example.moventaa.bean.moventaaendpoint.model.Provincia;
import com.example.moventaa.presenter.BusquedaCentrosPresenter;
import com.example.moventaa.presenter.IBusquedaCentrosView;

/**
 * Activity para la busqueda de centros
 * @author mmartinezced
 *
 */
public class BusquedaCentrosActivity extends Activity implements IBusquedaCentrosView{

	//Atributos
	List<Provincia> listProv = null;
	List<Localidad> listLoc = null;
	
	//Inyect pattern
	@InjectView(R.id.find_spinner_provincia) Spinner spinnerprov;
	@InjectView(R.id.btnBusqCentros) Button btnBuscar;
	@InjectView(R.id.find_spinner_localidad) Spinner spinnerlocalidad;
	
	
	BusquedaCentrosPresenter presenter;
	
	/**
	 *  Constructor de la pantalla
	 */
	 @Override
     protected void onCreate(Bundle savedInstanceState) {
	        super.onCreate(savedInstanceState);
	        setContentView(R.layout.activity_busquedacentros);
	        
	        ButterKnife.inject(this);
	        presenter = new BusquedaCentrosPresenter(this);
	        searchProvincias();
    }
	 
	 
	 
	@OnItemSelected(R.id.find_spinner_provincia) public void onItemSelected() {
		Object item = spinnerprov.getSelectedItem();
    	for (Provincia prov : listProv) {
			if(prov.getProvincia().equals(item)){
				searchLocalidades(prov.getKey().getId().toString());
				break;
			}
		}
	}
	
	
	 
	 /**
	  * Boton siguiente para ir a buscar centros
	  * @param view
	  */
    @OnClick(R.id.btnBusqCentros) public void onClickStock(View view){
    	
        String itemsel = (String)spinnerlocalidad.getSelectedItem();
        String idLocalidad="";
        if(listLoc instanceof List){
            for(Localidad loc : listLoc){
           	 if(loc.getLocalidad().equals(itemsel)){
           		 idLocalidad = loc.getKey().getName().toString();
           	 }
            }
        }   
        
        if(!idLocalidad.equals("")){
            Intent myIntent = new Intent(BusquedaCentrosActivity.this, ListCentrosActivity.class);
            myIntent.putExtra("idLocalidad",idLocalidad);
            startActivity(myIntent);
        }else{
       	 Toast.makeText(getBaseContext(),
					"Debe seleccionar una localidad",
					Toast.LENGTH_LONG).show();
        }
	}
	 
	 
	 
	/**
	 * Cargador de provincias en el spinner 
	 * @param list 
	 */
    public void loadSpinnerProvincias(List<Provincia> list) {
		listProv = list;
		List<String> provincias = new ArrayList<String>();
		for (Provincia provincia : list) {
			provincias.add(provincia.getProvincia());	
		}
		
    	// Create an ArrayAdapter using the string array and a default spinner layout
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(
						this, android.R.layout.simple_spinner_item, provincias);
		
    	// Specify the layout to use when the list of choices appears
    	adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	// Apply the adapter to the spinner
    	spinnerprov.setAdapter(adapter);
	}
	
	
	/**
	 * Cargador de provincias en el spinner 
	 * @param list 
	 */
	public void loadSpinnerLocalidades(List<Localidad> list) {
		listLoc = list;
		List<String> localidades = new ArrayList<String>();
		if(list instanceof List){
			for (Localidad localidad : list) {
				localidades.add(localidad.getLocalidad());	
			}
		}	
		
		// Create an ArrayAdapter using the string array and a default spinner layout
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(
						this, android.R.layout.simple_spinner_item, localidades);
		
    	// Specify the layout to use when the list of choices appears
    	adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	// Apply the adapter to the spinner
    	spinnerlocalidad.setAdapter(adapter);
	}


	@Override
	public void searchLocalidades(String idProvincia) {
		presenter.searchLocalidades(idProvincia);
		
	}

	@Override
	public void searchProvincias() {
		// TODO Auto-generated method stub
		presenter.searchProvincias();
	}
	
}
