package com.example.moventaa;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.TextView;
import android.widget.Toast;

import com.example.moventaa.bean.centroendpoint.Centroendpoint;
import com.example.moventaa.bean.centroendpoint.model.Centro;
import com.example.moventaa.bean.centroendpoint.model.CollectionResponseCentro;
import com.example.moventaa.utils.UserPrefsData;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;

/**
 * Activity para la busqueda de productos
 * 
 * @author mmartinezced
 *
 */
public class ListCentrosActivity extends ListActivity {
	
	//Progress dialog
	ProgressDialog progressDialog;
	CentrosRowAdapter adapter;
	
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_centros_list);

    	progressDialog = ProgressDialog.show(this,
    					 "", 
    				  	 "Bscando centros. Por favor, espere...",
    				  	 true,
    				  	 true);
        
    	final Button button = (Button) findViewById(R.id.bnt_centrosencontrados);
	      button.setOnClickListener(new View.OnClickListener() {
	           public void onClick(View v) {
	        	savedCentrosPreferences();
	           }
        });
    	
    	
        
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            String value = extras.getString("idLocalidad");
            new ListDataTaskRetriever().execute(value);    
        }
        
        

    }
    
    /**
     * Carga los datos en el listo row
     * @param list
     */
    private void fillRowsList(List<Centro> list){
    	adapter = new CentrosRowAdapter(this, list);
        setListAdapter(adapter);
        adapter.notifyDataSetChanged();
    }
    
    
    private void savedCentrosPreferences(){
   	    
    	List<Centro> centros = adapter.getSelected();
    	
    	if(centros.size()==0){
    		 Toast.makeText(getBaseContext(),
						"Debe al menos seleccionar un centro",
						Toast.LENGTH_LONG).show();
    	}else{
	   		SharedPreferences sharedPref = getSharedPreferences(
	   				MainActivity.PREFS_NAME, Context.MODE_PRIVATE);
	   		String serializedDataFromPreference = sharedPref.getString(
	   				MainActivity.PREFS_KEY, null);
	   		UserPrefsData restoredMyData = UserPrefsData
	   				.create(serializedDataFromPreference);
	
	   		List<String> data = new ArrayList<String>();
	   		for (Centro centro : centros) {
				data.add(centro.getKey().getName());
			}
	   		restoredMyData.setCentros(data);
	
	   		SharedPreferences.Editor editor = sharedPref.edit();
	   		editor.putString(MainActivity.PREFS_KEY, restoredMyData.serialize());
	   		editor.commit();
	   		
	   		Intent myIntent = new Intent(ListCentrosActivity.this, MainActivity.class);
	   		ListCentrosActivity.this.startActivity(myIntent);
    	}	
   }
    


	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * (e.g., a store)
	 */
	private class ListDataTaskRetriever extends AsyncTask<String, Void, CollectionResponseCentro> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CollectionResponseCentro doInBackground(String... params) {
			Centroendpoint.Builder builder = new Centroendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Centroendpoint endpoint = builder.build();
			try {
				Centro centro = new Centro(); 
				centro.setLocalidadID(params[0]);

				return endpoint.findCentroFilter(centro).execute();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CollectionResponseCentro result) {
       	  super.onPostExecute(result);
       	  progressDialog.dismiss();
       	  
		  if (result != null && result.getItems() != null && result.getItems().size() > 0) {
			  fillRowsList(result.getItems());
		  }else{
			  Toast.makeText(getBaseContext(),
						"No se han encontrado centros para la bsqueda",
						Toast.LENGTH_LONG).show();
		  }
	   }
	}  
    
	
	
   /**
    * Inner class con el adapatador de row de centros
    * 
    */
   class CentrosRowAdapter extends ArrayAdapter<Centro> {
		 
        private final Context context;
        private final List<Centro> itemsArrayList;
        
        private List<Centro> selected = new ArrayList<Centro>();  
        
 
        public CentrosRowAdapter(Context context, List<Centro> list) {
 
            super(context, R.layout.centros_row,list);
 
            this.context = context;
            this.itemsArrayList = list;
        }
        
        @Override
        public View getView(final int position, View convertView, ViewGroup parent) {
 
        	
//        	View v = convertView;
//			if (v == null) {
//				LayoutInflater vi = (LayoutInflater) this.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
//				v = vi.inflate(R.layout.userfilms_list_row, null);
//			}
        	
            //Create inflater 
            LayoutInflater inflater = (LayoutInflater) context
                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            
            View rowView = inflater.inflate(R.layout.centros_row, parent, false);
 
            TextView nombre = (TextView) rowView.findViewById(R.id.productosListTitle);
            TextView centrocp = (TextView) rowView.findViewById(R.id.centroCP);
            TextView direccion = (TextView) rowView.findViewById(R.id.centroDireccion);
            
 
            nombre.setText(itemsArrayList.get(position).getNombreCentro());
            direccion.setText(itemsArrayList.get(position).getDireccion());
            centrocp.setText(itemsArrayList.get(position).getCodigopostal());
            
            
       	    CheckBox checkbox = (CheckBox) rowView.findViewById(R.id.centro_checkbox);
       	    checkbox.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					String value = itemsArrayList.get(position).getKey().getName();
					if (((CheckBox) v).isChecked()) {
						selected.add(itemsArrayList.get(position));	
					}else{
						ListIterator<Centro> it = selected.listIterator();
						for(int i=0; it.hasNext();){
							Centro s = it.next();
							if(value.equals(s.getKey().getName())){
								selected.remove(s);
							}
						}
					}
				}
			});
       	    
            
            return rowView;
        }
        
        public Centro getItem(int position){
        	return null;
        }

		public List<Centro> getSelected() {
			return selected;
		}
	}







}
