package com.example.moventaa;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

import com.example.moventaa.bean.centroendpoint.Centroendpoint;
import com.example.moventaa.bean.centroendpoint.model.Centro;
import com.example.moventaa.bean.centroendpoint.model.CollectionResponseCentro;
import com.example.moventaa.bean.centroendpoint.model.Key;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.StringUtils;
import com.google.gson.Gson;

/**
 * Lista los centros asociados para el empleado
 * y que pueda selecionar cambiar de centro de trabajo
 * 
 * @author mmartinezced
 *
 */
public class ListCentrosUsuarioActivity extends ListActivity {
	
	//Progress dialog
	ProgressDialog progressDialog;
	
	CentrosRowAdapter adapter = null;
	
	
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_centros_list);

    	progressDialog = ProgressDialog.show(this,
    					 "", 
    				  	 "Bscando centros. Por favor, espere...",
    				  	 true,
    				  	 true);
    	
    	 Bundle extras = getIntent().getExtras();
         if (extras != null) {
             String value = extras.getString("listCentros");
             new ListDataTaskRetriever().execute(value);    
         }
         
         final Button button = (Button) findViewById(R.id.bnt_centrosencontrados);
	        button.setOnClickListener(new View.OnClickListener() {
	            public void onClick(View v) {
	            	checkSelected();
	            }
	       });    
        
    }
    
    
    private void checkSelected(){
    	if(adapter.getSelected()!=null && adapter.getSelected().length()>0){
    		final AppGlobal globalVariable = (AppGlobal) getApplicationContext();
            // Calling Application class (see application tag in AndroidManifest.xml)
            globalVariable.setCentroSelecID(adapter.getSelected());
    		this.finish();
	    }else{
	   	 Toast.makeText(getBaseContext(),
					"Debe seleccionar un centro de trabajo",
					Toast.LENGTH_LONG).show();
	    }
    }
    
    
    /**
     * Carga los datos en el listo row
     * @param list
     */
    private void fillRowsList(List<Centro> list){
    	adapter = new CentrosRowAdapter(this, list);
        setListAdapter(adapter);
        adapter.notifyDataSetChanged();
    }
    


	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * (e.g., a store)
	 */
	private class ListDataTaskRetriever extends AsyncTask<String, Void, CollectionResponseCentro> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CollectionResponseCentro doInBackground(String... params) {
			Centroendpoint.Builder builder = new Centroendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Centroendpoint endpoint = builder.build();
			
				CollectionResponseCentro lst = new CollectionResponseCentro();
				lst.setItems(new ArrayList<Centro>());
				
				List<String> list = new Gson().fromJson(params[0], ArrayList.class);
				for (String string : list) {
					Centro content = new Centro();
					Key key = new Key();
					key.setName(string);
					content.setKey(key);
					CollectionResponseCentro result;
					try {
						result = endpoint.findCentroFilter(content).execute();
						lst.getItems().addAll(result.getItems());
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				return lst;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CollectionResponseCentro result) {
       	  super.onPostExecute(result);
       	  progressDialog.dismiss();
       	  
		  if (result != null && result.getItems() != null && result.getItems().size() > 0) {
			  fillRowsList(result.getItems());
		  }else{
			  Toast.makeText(getBaseContext(),
						"No se han encontrado centros para la bsqueda",
						Toast.LENGTH_LONG).show();
		  }
	   }
	}  
    
	
	
   /**
    * Inner class con el adapatador de row de centros
    * 
    */
   class CentrosRowAdapter extends ArrayAdapter<Centro> {
		 
        private final Context context;
        private final List<Centro> itemsArrayList;
        
        private String selected = "";
        int selectedPosition = -1;
 
        public CentrosRowAdapter(Context context, List<Centro> list) {
 
            super(context, R.layout.centros_row,list);
 
            this.context = context;
            this.itemsArrayList = list;
        }
        
        @Override
        public View getView(final int position, View convertView, ViewGroup parent) {
 
        	
//        	View v = convertView;
//			if (v == null) {
//				LayoutInflater vi = (LayoutInflater) this.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
//				v = vi.inflate(R.layout.userfilms_list_row, null);
//			}
        	
            //Create inflater 
            LayoutInflater inflater = (LayoutInflater) context
                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            
            View rowView = inflater.inflate(R.layout.centros_usuario_row, parent, false);
 
            TextView nombre = (TextView) rowView.findViewById(R.id.productosListTitle);
            TextView centrocp = (TextView) rowView.findViewById(R.id.centroCP);
            TextView direccion = (TextView) rowView.findViewById(R.id.centroDireccion);
            
 
            nombre.setText(itemsArrayList.get(position).getNombreCentro());
            direccion.setText(itemsArrayList.get(position).getDireccion());
            centrocp.setText(itemsArrayList.get(position).getCodigopostal());
            
            
            RadioButton radio = (RadioButton)rowView.findViewById(R.id.userCentroSelect);
            radio.setChecked(position == selectedPosition);
            radio.setTag(position);
            radio.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					selected = itemsArrayList.get(position).getKey().getName();
					selectedPosition = (Integer)v.getTag();
					notifyDataSetInvalidated();
				}
			});
            return rowView;
        }

		public String getSelected() {
			return selected;
		}
	}    
}
