package com.example.moventaa;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import android.app.DownloadManager;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Toast;

import com.example.moventaa.bean.moventaaendpoint.model.Nominas;
import com.example.moventaa.presenter.INominasView;
import com.example.moventaa.presenter.NominasPresenter;

/**
 * Activity para el listado de nominas
 * 
 * @author mmartinezced
 *
 */
public class ListNominasActivity extends ListActivity implements INominasView, OnItemClickListener{
	
	//Progress dialog
	ProgressDialog progressDialog;
	//Presneter
    NominasPresenter presenter; 
    
	
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_nominas_list);
        this.presenter = new NominasPresenter(this);

    	progressDialog = ProgressDialog.show(this,
    					 "", 
    				  	 "Bscando nominas. Por favor, espere...",
    				  	 true,
    				  	 true);
    	
    	getListView().setOnItemClickListener(this);
    	loadNominas();
    }
    

	@Override
	public void fillRowsList(List<Nominas> items) {
		List<String> fechas = new ArrayList<String>();
		Calendar calendar = Calendar.getInstance();
	    for (Nominas nominas : items) {
	       calendar.setTimeInMillis(nominas.getDate().getValue());
	       calendar.add(Calendar.MONTH, 1);
		   fechas.add(new SimpleDateFormat("MMM-yyyy").format(calendar.getTime()));
		}
	     ArrayAdapter adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, fechas);
		 setListAdapter(adapter);
		 adapter.notifyDataSetChanged();
		
	}

	@Override
	public void loadNominas() {
		AppGlobal globalVariable = (AppGlobal) getApplicationContext();
    	String idempleado = globalVariable.getEmpleadoID();
    	
    	if(idempleado == null){
    		progressDialog.dismiss();
			  Toast.makeText(getBaseContext(),
						"No hay un empleado registrado. Antes debe registrarse",
						Toast.LENGTH_LONG).show();
    		
    	}else{
    		presenter.searchNominas(idempleado);
    	}	
	}

	@Override
	public void descargaNomina(String mes) {
		AppGlobal globalVariable = (AppGlobal) getApplicationContext();
    	String idempleado = globalVariable.getEmpleadoID();
    	
		// get download service and enqueue file
		DownloadManager manager = (DownloadManager) getSystemService(Context.DOWNLOAD_SERVICE);
		presenter.descargaNomina(idempleado, mes, manager);
	}



	@Override
	public void showProgress() {
		progressDialog.show();
	}

	@Override
	public void hideProgress() {
		progressDialog.dismiss();
	}


	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position,	long id) {
		String object = (String)parent.getItemAtPosition(position);
		int month;
		try {
			month = new SimpleDateFormat("MMM-yyyy").parse(object).getMonth()+1;
			descargaNomina(Integer.toString(month));  
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}  
}
