package com.example.moventaa;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.StrictMode;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AnimationUtils;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.NumberPicker;
import android.widget.TextView;
import android.widget.Toast;

import com.example.moventaa.bean.ventaproductocentroendpoint.Ventaproductocentroendpoint;
import com.example.moventaa.bean.ventaproductocentroendpoint.model.EmpleadoProductoVentas;
import com.example.moventaa.bean.ventaproductocentroendpoint.model.Producto;
import com.example.moventaa.bean.ventaproductocentroendpoint.model.VentaProductoCentro;
import com.example.moventaa.utils.ImageDownloader;
import com.example.moventaa.utils.Utils;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;

/**
 * Activity para el listado de productos 
 * asociados al empleado
 * 
 * @author mmartinezced
 *
 */
public class ListProductosFragment extends android.support.v4.app.ListFragment 
                                             implements ListProductFragmentInterface {

	public ListView mList;
	boolean mListShown;
	View mProgressContainer;
	ProductosRowAdapter rowadapter = null; 
	
	  static{
	     StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();  
	     StrictMode.setThreadPolicy(policy);  
	  }
	
	 @Override
	 public View onCreateView(LayoutInflater inflater, ViewGroup container,
	           Bundle savedInstanceState) {

	      View rootView = inflater.inflate(R.layout.fragment_top_productos, container, false);
	      
	      mList = (ListView) rootView.findViewById(android.R.id.list);
		  mProgressContainer = rootView.findViewById(R.id.progressContainer);
		  mListShown = true;
	      
		  setListShown(false);
	      return rootView;
	 }
	 
	 
	 
	 
	 public void onActivityCreated(Bundle savedInstanceState) {
		    super.onActivityCreated(savedInstanceState);

		    final AppGlobal globalVariable = (AppGlobal) getActivity().getApplicationContext();
		    if(globalVariable.getCentroSelecID()!=null){
		    	LisDataTaskRetriever retriever = new LisDataTaskRetriever();
		    	retriever.execute(globalVariable.getCentroSelecID(),
		    			                 globalVariable.getEmpleadoID());
		    }	
	 }
	 
	 
	 private void loadDataAdapter(EmpleadoProductoVentas result){
		 rowadapter = new ProductosRowAdapter(getActivity(), 
				 							  result.getProductos(),
				 							  result.getProductosVendidos(),
				 							  result.getProductoStock());
		 setListAdapter(rowadapter);
		 rowadapter.notifyDataSetChanged();
		 setListShown(true);
	 }
	 
	 
	 /**
	  * LLamada al servicio web que se encarga de la gestion de la venta realziada por el empleado
	  */
	 private void ventaProducto(int cItems, String idProducto){
		 
		 final AppGlobal globalVariable = (AppGlobal) getActivity().getApplicationContext();
		 
		 DataTaskSender senderventa = new DataTaskSender();
		 senderventa.execute(globalVariable.getCentroSelecID(),
				 					globalVariable.getEmpleadoID(),
				 					idProducto,
				 					Integer.toString(cItems));
	
		 LisDataTaskRetriever retriever = new LisDataTaskRetriever();
	   	 retriever.execute(globalVariable.getCentroSelecID(),
	   			           globalVariable.getEmpleadoID());
	 }
	 
	 
	 public void setListShown(boolean shown, boolean animate){
		    if (mListShown == shown) {
		        return;
		    }
		    mListShown = shown;
		    if (shown) {
		        if (animate) {
		            mProgressContainer.startAnimation(AnimationUtils.loadAnimation(
		                    getActivity(), android.R.anim.fade_out));
		        }
		        mProgressContainer.setVisibility(View.GONE);
		    } else {
		        if (animate) {
		            mProgressContainer.startAnimation(AnimationUtils.loadAnimation(
		                    getActivity(), android.R.anim.fade_in));
		        }
		        mProgressContainer.setVisibility(View.VISIBLE);
		    }
	}
	public void setListShown(boolean shown){
		    setListShown(shown, true);
	}
	public void setListShownNoAnimation(boolean shown) {
	    setListShown(shown, false);
	}
	 
	 
	 
	 /**
	  * Asyntask de venta de producto
	  * @author mmartinezced
	  *
	  */
	 private class DataTaskSender extends AsyncTask<String, Void, Void> {

		 
		 protected Void doInBackground(String... params) {
			
			Ventaproductocentroendpoint.Builder builder = new Ventaproductocentroendpoint.Builder(
		            AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
		            null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			
			Ventaproductocentroendpoint endpoint = builder.build();
			try {
				for(int i=0; i<Integer.parseInt(params[3]);i++){
					VentaProductoCentro venta = new VentaProductoCentro();
					venta.setCentroId(params[0]);
					venta.setEmpleadoID(params[1]);
					venta.setProductoId(params[2]);
					endpoint.insertVentaProductoCentro(venta).execute();
				}
		        
			} catch (IOException e) {
		          // TODO Auto-generated catch block
		          e.printStackTrace();
		    }
			return null;
		 }

		@Override
		protected void onPostExecute(Void result) {
			// TODO Auto-generated method stub
			super.onPostExecute(result);
			Context context = getActivity().getApplicationContext();
	        Toast.makeText(context, "Venta enviada...", Toast.LENGTH_LONG).show();	
		}
	 }
	 
	 
	 /**
	  * Obtiene el listado de ventas por producto y por centro
	  * @author mmartinezced
	  *
	  */
	private class LisDataTaskRetriever extends
			AsyncTask<String, Void, EmpleadoProductoVentas> {

		@Override
		protected EmpleadoProductoVentas doInBackground(String... params) {

			Ventaproductocentroendpoint.Builder builder = new Ventaproductocentroendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);

			builder = CloudEndpointUtils.updateBuilder(builder);

			Ventaproductocentroendpoint endpoint = builder.build();

			try {
				EmpleadoProductoVentas lst = endpoint.getEmpleadoVentasProductoCentro()
						.setIdCentro(params[0])
						.setIdEmpleado(params[1])
						.execute();
				return lst;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
		}

		@SuppressWarnings("null")
		protected void onPostExecute(EmpleadoProductoVentas result) {
			super.onPostExecute(result);
			if (result != null) {
				loadDataAdapter(result);
			}
		}
	}
	 

	
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		// TODO Auto-generated method stub
		super.onListItemClick(l, v, position, id);
		
		Producto prodsel = rowadapter.getItem(position);
		if(prodsel instanceof Producto){
			Intent myIntent = new Intent(ListProductosFragment.this.getActivity(), ProductoInfoActivity.class);
			myIntent.putExtra(ProductoInfoActivity.INTENT_EXTRA,prodsel.getKey().getName());
			ListProductosFragment.this.startActivity(myIntent);
		}	
	}   
	
	
	/**
	 * Inner class con el adapatador de row de centros
	 * 
	 */
	class ProductosRowAdapter extends ArrayAdapter<Producto> {

		private final Context context;
		private final List<Producto> itemsArrayList;
		private Map<String,BigDecimal> ventas = new HashMap<String, BigDecimal>();
		private Map<String,BigDecimal> stock = new HashMap<String, BigDecimal>();
		
		//utilidad para cargar imagenes
		private ImageDownloader imagedownloader = new ImageDownloader();

		
		public ProductosRowAdapter(Context listProductosFragment, List<Producto> list, Map ventas , Map stock) {
			super(listProductosFragment, R.layout.productos_row, list);

			this.context = listProductosFragment;
			this.itemsArrayList = list;
			if(ventas != null){
				this.ventas = ventas;
			}
			this.stock = stock;
		}

		@Override
		public View getView(final int position, View convertView, ViewGroup parent) {
			
			// Create inflater
			LayoutInflater inflater = (LayoutInflater) context
					.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			View rowView = inflater.inflate(R.layout.productos_row, parent, false);
			
			TextView titulo = (TextView) rowView.findViewById(R.id.row_prod_titulo);
			titulo.setText(itemsArrayList.get(position).getNombre());
			
			
			TextView prodvendidos = (TextView) rowView.findViewById(R.id.rowprodvendidosnumber);
			BigDecimal value = (BigDecimal)ventas.get(itemsArrayList.get(position).getKey().getName());
			prodvendidos.setText(value == null ? "00" : "" + value.intValue());
			
			
			ImageView image = (ImageView) rowView.findViewById(R.id.row_prod_image);
			//new ImageDownloader().download(,image);
			imagedownloader.download(Utils.getProductsUrlImage( itemsArrayList.get(position).getCodigoBR()), image);
			
			TextView stockTx = (TextView) rowView.findViewById(R.id.prod_stock);
			BigDecimal valuestock = (BigDecimal)stock.get(itemsArrayList.get(position).getKey().getName());
			stockTx.setText(valuestock == null ? "00" : "" + valuestock.intValue());
			
			
			
			TextView btn = (TextView) rowView.findViewById(R.id.rowprodbtnvender);
			btn.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					NumberPickerDialog myDiag = new NumberPickerDialog(itemsArrayList.get(position).getNombre(),
																	   itemsArrayList.get(position).getKey().getName());
					myDiag.show(getFragmentManager(),"number");
				}
			});
			
			return rowView;
		}
		
		
		public Producto getItem(int position){
			return itemsArrayList.get(position); 
		}
	}
	
	/**
	 * Inner class number dialog
	 * @author mmartinezced
	 *
	 */
	class NumberPickerDialog extends android.support.v4.app.DialogFragment {

		private String producto;
		private String idProducto;
		
		public NumberPickerDialog(String producto, String idProducto) {
			super();
			this.producto = producto;
			this.idProducto = idProducto;
		}
		
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			
		    AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		    // Get the layout inflater
		    LayoutInflater inflater = getActivity().getLayoutInflater();

		    View v1 = inflater.inflate(R.layout.numberpicker_dialog, null);

		    final NumberPicker picker = (NumberPicker) v1.findViewById(R.id.pref_num_picker );
		    picker.setMinValue(1);
		    picker.setMaxValue(100);
		    picker.setWrapSelectorWheel(false);
		    
		    
		    // Inflate and set the layout for the dialog
		    // Pass null as the parent view because its going in the dialog layout
		    builder.setView(v1);
		    
		    builder.setMessage(R.string.dialog_number_message)
		       	   .setTitle(R.string.dialog_number_title);
		           //.setMessage(R.string.dialog_fire_missiles)
		    // Add action buttons
		    builder.setPositiveButton(R.string.dialog_number_btn_accept, new DialogInterface.OnClickListener() {
//		               @Override
		               public void onClick(DialogInterface dialog, int id) {
		            	    int pickedValue = picker.getValue();
		            	    
		            	    ventaProducto(pickedValue, idProducto);
		            	    
		                    // set your TextView id instead of R.id.textView1
		                    //TextView textView = (TextView)findViewById(R.id.textView1);
		                    //textView.setText(Integer.toString(pickedValue));
		               }
		           })
		           .setNegativeButton(R.string.dialog_number_btn_cancel, new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		                   //LoginDialogFragment.this.getDialog().cancel();
		               }
		           });      
		    return builder.create();
		}
	}



	@Override
	public void update() {
		 final AppGlobal globalVariable = (AppGlobal) getActivity().getApplicationContext();
		    if(globalVariable.getCentroSelecID()!=null){
		    	LisDataTaskRetriever retriever = new LisDataTaskRetriever();
		    	retriever.execute(globalVariable.getCentroSelecID(),
		    			                 globalVariable.getEmpleadoID());
		    }	
	}


	 
}
