package com.example.moventaa;

import java.io.IOException;
import java.util.List;

import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.ActionBar.TabListener;
import android.app.AlertDialog;
import android.app.FragmentTransaction;
import android.app.SearchManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.Settings;
import android.support.v4.app.FragmentActivity;
import android.support.v4.view.MenuItemCompat;
import android.support.v4.view.ViewPager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.SearchView;
import android.widget.TextView;
import android.widget.Toast;

import com.example.moventaa.bean.centroendpoint.Centroendpoint;
import com.example.moventaa.bean.centroendpoint.model.Centro;
import com.example.moventaa.bean.centroendpoint.model.CollectionResponseCentro;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint.FindCentroByCoords;
import com.example.moventaa.bean.moventaaendpoint.model.CentroCollection;
import com.example.moventaa.bean.moventaaendpoint.model.Nominas;
import com.example.moventaa.utils.UserPrefsData;
import com.example.moventaa.utils.Utils;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.gson.Gson;
import com.google.zxing.client.android.CaptureActivity;

/**
 * Main activity de arranque de la aplicacion
 *
 * @author mmartinezced
 */
public class MainActivity extends FragmentActivity 
                           implements TabListener {

	public static final int REQUEST_CODE = 0x0000c0de; // Only use bottom 16 bits
	
	//fichero de preferencias
	public static final String PREFS_NAME = "MyMoventaaPrefsFile";
	public static final String PREFS_KEY = "MyMoventaaPrefsKey";
	
	
	private ViewPager viewPager;
    private TabsPagerAdapter mAdapter;
    private ActionBar actionBar;
    
    //location listener para el gps
    private LocationListener locationListener;
    
    // Tab titles
    //private String[] tabs = { "Productos", "Ventas Semanal", "Ventas Mensual" };
    private String[] tabs = { "Productos" };

    
	
    @Override
    protected void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
    
        
        //Comprueba si hay conexion a internet
        checkInternetConnection();
        UserPrefsData userpref = checkRegisterEmployed();
        checkCentrosGPS(userpref);
        
        
        // get action bar   
        actionBar = getActionBar();
        
        // Enabling Up / Back navigation
        //actionBar.setDisplayHomeAsUpEnabled(true);
        
        
        // Initilization
        viewPager = (ViewPager) findViewById(R.id.pager);
        mAdapter = new TabsPagerAdapter(getSupportFragmentManager());
 
        viewPager.setAdapter(mAdapter);
        actionBar.setHomeButtonEnabled(false);
        //actionBar.setNavigationMode(ActionBar.DISPLAY_SHOW_TITLE);
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
        
        
        // Adding Tabs
        for (String tab_name : tabs) {
            actionBar.addTab(actionBar.newTab().setText(tab_name)
                    .setTabListener(this));
        }
    }



	@Override
	public void onTabSelected(Tab tab, FragmentTransaction ft) {
	}


	@Override
	public void onTabUnselected(Tab tab, FragmentTransaction ft) {
	}


	@Override
	public void onTabReselected(Tab tab, FragmentTransaction ft) {
	}

   


	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.activity_main_actions, menu);
    	
        MenuItem searchItem = menu.findItem(R.id.action_search);
        
        
        // Associate searchable configuration with the SearchView
        SearchManager searchManager =
               (SearchManager) getSystemService(Context.SEARCH_SERVICE);
//        SearchView searchView =
//                (SearchView) menu.findItem(R.id.action_search).getActionView();
        SearchView searchView = (SearchView)MenuItemCompat.getActionView(searchItem);
        
        // Assumes current activity is the searchable activity
        if (searchView == null) {
            MenuItemCompat.setShowAsAction(searchItem, MenuItem.SHOW_AS_ACTION_COLLAPSE_ACTION_VIEW|MenuItem.SHOW_AS_ACTION_ALWAYS);
            MenuItemCompat.setActionView(searchItem, searchView = new SearchView(getBaseContext()));
        }
        searchView.setSearchableInfo(
                searchManager.getSearchableInfo(getComponentName()));
        	
        // Get textview id of search widget
        int id =  searchView.getContext().getResources().getIdentifier("android:id/search_src_text", null, null);
        TextView textView = (TextView) searchView.findViewById(id);

        // Set search text color
        textView.setTextColor(Color.WHITE);

        // Set search hints color
        textView.setHintTextColor(Color.GRAY);
        
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.action_cambiocentro) {
        	
        	UserPrefsData restoredMyData = Utils.getUserPrefs(this);
        	
        	Intent myIntent = new Intent(MainActivity.this, ListCentrosUsuarioActivity.class);
            myIntent.putExtra("listCentros",new Gson().toJson(restoredMyData.getCentros()));
            MainActivity.this.startActivityForResult(myIntent, REQUEST_CODE);
            //MainActivity.this.finishActivity(requestCode);
            return true;
        }else if (id == R.id.action_empleado){
        	openRegisterEmployeed(null);
        }else if (id == R.id.action_barcode){
        	openBarCodeScanner();
        }else if (id == R.id.action_vacaciones){
        	openVacaciones();
        }else if (id == R.id.action_bajas){
        	openBajas();
        }else if (id== R.id.action_nominas){
        	Intent myIntent = new Intent(MainActivity.this, ListNominasActivity.class);
    		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
    		MainActivity.this.startActivity(myIntent);
        }
        
        return super.onOptionsItemSelected(item);
    }


	private void openVacaciones() {
		Intent myIntent = new Intent(MainActivity.this, VacacionesActivity.class);
		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		MainActivity.this.startActivity(myIntent);
	}

	private void openBajas(){
		Intent myIntent = new Intent(MainActivity.this, BajaActivity.class);
		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		MainActivity.this.startActivity(myIntent);
	}
	
	

	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		 super.onActivityResult(requestCode, resultCode, data);
		 if (requestCode == REQUEST_CODE) {
			 mAdapter.notifyDataSetChanged();
		 }
	}
	
	
	/**
	 * Comprueba los diferentes parametros de arranque
	 * @return
	 */
	private UserPrefsData checkRegisterEmployed(){
		
		// Restore preferences
        SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);
        String serializedDataFromPreference = settings.getString(PREFS_KEY, null);
        
        
        //para preubas
        //serializedDataFromPreference = null;
        
        UserPrefsData restoredMyData = null;
        if(serializedDataFromPreference==null){
            alertDialogRegEmployed();
        }else{
        	
            restoredMyData = UserPrefsData.create(serializedDataFromPreference);
            //No esta registrado, lo registramos
            if(!restoredMyData.getRegistred()){
            	Intent myIntent = new Intent(MainActivity.this, EmployedActivity.class);
            	MainActivity.this.startActivity(myIntent);
            
            //Esta registrado
            }else if(restoredMyData.getRegistred()){
            	
            	AppGlobal globalVariable = (AppGlobal) getApplicationContext();
            	// Calling Application class (see application tag in AndroidManifest.xml)
            	globalVariable.setEmpleadoID(restoredMyData.getIdEmpleado());
            	return restoredMyData;
            }
            
          
        }
	
        return restoredMyData;
	}
	
	
	/**
	 * Metodo que comprueba que si estando el usuario
	 * registrado, este tiene un centro o varios centros
	 * asociados, en el caso de ser plural, detecta si 
	 * tiene gps activado en caso negativo le mustra 
	 * una pantalla de listado de centros para que seleccione
	 * uno
	 * 
	 * @param userpref
	 */
	private void checkCentrosGPS(UserPrefsData userpref) {

    	AppGlobal globalVariable = (AppGlobal) getApplicationContext();
		if(userpref!=null 
			&& userpref.getCentros().size()>0
		    && globalVariable.getCentroSelecID() == null){
			
			
			if(userpref.getGpsenabled()){
				LocationManager service = (LocationManager) getSystemService(LOCATION_SERVICE);
		    	boolean enabled = service.isProviderEnabled(LocationManager.GPS_PROVIDER);
		    	
		    	if (!enabled) {
		    	   Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
		      	   startActivity(intent);
		    	}else{
		    		defineLocationListener(service);
		    	}
			}else{
			   //Esta registrado y tiene mas de un centro y no activo el gps
			   Intent myIntent = new Intent(MainActivity.this, ListCentrosUsuarioActivity.class);
			   myIntent.putExtra("listCentros",new Gson().toJson(userpref.getCentros()));
			   MainActivity.this.startActivityForResult(myIntent,REQUEST_CODE);
			}
		}else if(userpref!=null &&
				 userpref.getCentros().size()==0){
			 Intent myIntent = new Intent(this, BusquedaCentrosActivity.class);
			 startActivity(myIntent);
		}
    }
	
	
	private void defineLocationListener(LocationManager service ){
		// Define a listener that responds to location updates
		 locationListener = new LocationListener() {
		    public void onLocationChanged(Location location) {
		      // Called when a new location is found by the network location provider.
		      //makeUseOfNewLocation(location);
		    	GPSCentroTaskRetriever gpsretrev = new GPSCentroTaskRetriever();
		    	gpsretrev.execute(location);
		    }
		    
		    public void onStatusChanged(String provider, int status, Bundle extras) {}
		    public void onProviderEnabled(String provider) {}
		    public void onProviderDisabled(String provider) {}
		  };

		  String locationProvider = LocationManager.GPS_PROVIDER;
	  	  // Register the listener with the Location Manager to receive location updates
		  service.requestLocationUpdates(locationProvider, 0, 0, locationListener);
	}
	
	/**
	 * Elimina el listener del gps si encuentra un centro asociado
	 * @param list
	 */
	private void removeLocationListener(List<com.example.moventaa.bean.moventaaendpoint.model.Centro> list){
		
		if(list.size()>0){
			AppGlobal globalVariable = (AppGlobal) getApplicationContext();
			com.example.moventaa.bean.moventaaendpoint.model.Centro centro = list.get(0);
			globalVariable.setCentroSelecID(centro.getKey().getName());
			
			LocationManager service = (LocationManager) getSystemService(LOCATION_SERVICE);
			service.removeUpdates(locationListener);
			locationListener = null;
			
			
			viewPager.getAdapter().notifyDataSetChanged(); 
		}	
	}
	
	/**
	 * Llama al activity relacionado con la responsabilidad
	 * de activar el barcodescanner
	 */
	private void openBarCodeScanner(){
		Intent myIntent = new Intent(this, ProductoInfoActivity.class);
		myIntent.setAction("BARCODESCANNER");
		startActivity(myIntent);
	}	
	
	
	private void openRegisterEmployeed(DialogInterface dialog){
		if(dialog!=null){
			dialog.dismiss();
		}	
		Intent myIntent = new Intent(MainActivity.this, EmployedActivity.class);
		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		MainActivity.this.startActivity(myIntent);
	}
	
	
	/**
	 * Muestra un alert dialog informando que no hay empleado registrado y 
	 * que hay que registrar uno
	 */
	private void alertDialogRegEmployed() {

		AlertDialog.Builder builder = new AlertDialog.Builder(this);

		builder.setMessage(R.string.alertdlg_noemployed_message)
		       .setTitle(R.string.alertdlg_noemployed_title);

		builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
	           public void onClick(DialogInterface dialog, int id) {
	        	   openRegisterEmployeed(dialog);
	           }
	       });
		builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
	           public void onClick(DialogInterface dialog, int id) {
	               // User cancelled the dialog
	           }
	       });
		AlertDialog dialog = builder.create();
		//dialog.getWindow().setType(WindowManager.LayoutParams.TYPE_SYSTEM_ALERT);
	    dialog.show();
	    
	}


	/**
	 * Comprueba la hay conexion a internet
	 *  
	 * @return
	 */
	private boolean checkInternetConnection() {
		    ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		    // test for connection
		    if (cm.getActiveNetworkInfo() != null
		            && cm.getActiveNetworkInfo().isAvailable()
		            && cm.getActiveNetworkInfo().isConnected()) {
		        return true;
		    } else {
		    	Toast.makeText(this, "Internet Connection Not Present", Toast.LENGTH_LONG).show();
		        Log.v("c", "Internet Connection Not Present");
		        return false;
		    }
	}


	
	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * 
	 */
	private class GPSCentroTaskRetriever extends AsyncTask<Location, Void, CentroCollection> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CentroCollection doInBackground(Location... params) {
			Moventaaendpoint.Builder builder = new Moventaaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Moventaaendpoint endpoint = builder.build();
			try {
				double latitude = params[0].getLatitude();
		    	double longitude = params[0].getLongitude();
		    	
				CentroCollection result = endpoint.
						    findCentroByCoords(Double.toString(longitude),
						    				   Double.toString(latitude),
						                       Long.parseLong("1"), 1).execute();
				return result;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CentroCollection result) {
       	  super.onPostExecute(result);
      // 	  progressDialog.dismiss();
       	  
		  if (result != null && result.getItems() != null && result.getItems().size() > 0) {
			  removeLocationListener(result.getItems());
		  }
	   }
	}    
    

}
