package com.example.moventaa;

import java.io.IOException;

import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.app.ProgressDialog;
import android.app.SearchManager;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.webkit.WebView;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.NumberPicker;
import android.widget.TextView;
import android.widget.Toast;
import butterknife.ButterKnife;
import butterknife.InjectView;
import butterknife.OnClick;

import com.example.moventaa.R;
import com.example.moventaa.bean.moventaaendpoint.model.ProductoInfo;
import com.example.moventaa.presenter.IProductoInfoView;
import com.example.moventaa.presenter.ProductoInfoPresenter;
import com.example.moventaa.utils.ImageDownloader;
import com.example.moventaa.utils.Utils;
import com.google.zxing.client.android.CaptureActivity;
import com.google.zxing.client.android.Intents;

/**
 * Activity que implementa la muestra de informacion 
 * de un producto
 * 
 * @author mmartinezced
 */
public class ProductoInfoActivity extends Activity implements IProductoInfoView {

	//progress dialog
	private ProgressDialog progressDialog;
		
	public static final String INTENT_EXTRA = "productoID";
	public static final int REQUEST_CODE = 0x0000c0de; // Only use bottom 16 bits
	
	//utilidad para cargar imagenes
	private ImageDownloader imagedownloader = new ImageDownloader();
	
	private String idProducto;
	@InjectView(R.id.row_prod_titulo) TextView titulo;
	@InjectView(R.id.prodinfo_stock) TextView stock;
	@InjectView(R.id.prodinfo_pend) TextView pendientes;
	@InjectView(R.id.prodinfo_comentarios) TextView comentarios;
	@InjectView(R.id.prodinfoimage) ImageView imagen;
	@InjectView(R.id.textView3) WebView webview;
	@InjectView(R.id.btnsolictarstock) Button btnsolictarstock;
	
	//presenter
	private ProductoInfoPresenter presenter;
	private String idCentroSelec;
	private String idEmpleado;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_product_info);
		
		ButterKnife.inject(this);
		
		presenter = new ProductoInfoPresenter(this);
		
		final AppGlobal globalVariable = (AppGlobal) getApplicationContext();
		idCentroSelec = globalVariable.getCentroSelecID();
		idEmpleado = globalVariable.getEmpleadoID();
		
		progressDialog = ProgressDialog.show(this,
				                             "", 
			  	                             "Bscando datos. Por favor, espere...",
			  	                             true);
		
		Intent intent = getIntent();
	    if ("BARCODESCANNER".equals(intent.getAction())) {
	      String query = intent.getStringExtra(SearchManager.QUERY);
	      openBarCodeScanner();
	    }else{
			String idproducto = intent.getStringExtra(INTENT_EXTRA);
			presenter.loadProducto(idproducto, idCentroSelec);
	    }
	}
	
	
	private void openBarCodeScanner(){
		Intent myIntent = new Intent(this, CaptureActivity.class);
		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		myIntent.putExtra("SCAN_FORMATS", "EAN_8,EAN_13");
		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		myIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);
	    //attachMoreExtras(myIntent);
		startActivityForResult(myIntent, REQUEST_CODE);
	}
	
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
	   super.onActivityResult(requestCode, resultCode, data);
	   
	    if (requestCode == REQUEST_CODE) {
	    	if (resultCode == RESULT_OK) {
	    	  String barcode = (String)data.getExtras().get(Intents.Scan.RESULT);
	    	  presenter.loadProductoByBarCode(barcode, idCentroSelec);
	    	}else{
	    		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT);
	    		this.finish();
	    	}
	    }
	}
	
	@OnClick(R.id.btnsolictarstock)
	public void onClickStock(View view){
		NumberPickerDialog myDiag = new NumberPickerDialog(titulo.getText().toString(),
														   idProducto);
		myDiag.show(getFragmentManager(),"dialog");
	}
	
	
	/**
	 * Inner class number dialog
	 * @author mmartinezced
	 *
	 */
	class NumberPickerDialog extends DialogFragment {

		private String producto;
		private String idProducto;
		
		public NumberPickerDialog(String producto, String idProducto) {
			super();
			this.producto = producto;
			this.idProducto = idProducto;
		}
		
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			
		    AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		    // Get the layout inflater
		    LayoutInflater inflater = getActivity().getLayoutInflater();

		    View v1 = inflater.inflate(R.layout.numberpicker_dialog, null);

		    final NumberPicker picker = (NumberPicker) v1.findViewById(R.id.pref_num_picker );
		    picker.setMinValue(1);
		    picker.setMaxValue(100);
		    picker.setWrapSelectorWheel(false);
		    
		    
		    // Inflate and set the layout for the dialog
		    // Pass null as the parent view because its going in the dialog layout
		    builder.setView(v1);
		    
		    builder.setMessage(R.string.dialog_number_stock_desc)
		       	   .setTitle(R.string.dialog_number_stock_title);
		    // Add action buttons
		    builder.setPositiveButton(R.string.dialog_number_btn_accept, new DialogInterface.OnClickListener() {
//		               @Override
		               public void onClick(DialogInterface dialog, int id) {
		            	    int pickedValue = picker.getValue();
		            	    saveStockSolicitado(idProducto,idCentroSelec,Integer.toString(pickedValue),idEmpleado);
		               }
		           })
		           .setNegativeButton(R.string.dialog_number_btn_cancel, new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		               }
		           });      
		    return builder.create();
		}
	}
	
	public void showSolicitudStockEnviada(){
		  Toast.makeText(getBaseContext(),
					"Solicitud de stock enviada",
					Toast.LENGTH_LONG).show();
	}
	

	/**
	 * Carga los datos en la view
	 */
	@Override
	public void showProductoDataDetails(ProductoInfo productoinfo) {
		if(productoinfo==null){
			  Toast.makeText(getBaseContext(),
						"No se han encontrado datos...!!",
						Toast.LENGTH_LONG).show();
		}else{
			
			com.example.moventaa.bean.moventaaendpoint.model.Producto producto = productoinfo.getProducto();
			titulo.setText(producto.getNombre());
			idProducto = producto.getKey().getName();
			comentarios.setText(producto.getDescripcion());
			
			stock.setText(productoinfo.getStock());
			pendientes.setText(productoinfo.getSolicitado()==null ? "0" : productoinfo.getSolicitado());
			
				
			HttpGet httpRequest = new HttpGet(Utils.getProductsUrlInfo(producto.getCodigoBR()));
			HttpEntity httpEntity = null;
			HttpClient httpclient = new DefaultHttpClient();
			org.apache.http.HttpResponse response;
			try {
				response = httpclient.execute(httpRequest);
				if (response.getStatusLine().getStatusCode() == 200) {
					webview.loadUrl(Utils.getProductsUrlInfo(producto.getCodigoBR()));
				} else {
					webview.loadData(getString(R.string.html_error), "text/html",
							null);
				}
			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}		 
			
			imagedownloader.download(Utils.getProductsUrlImage(producto.getCodigoBR()),imagen);
		}
	}


	@Override
	public void showProgress() {
		progressDialog.show();
	}


	@Override
	public void hideProgress() {
		progressDialog.hide();
	}

	@Override
	public void saveStockSolicitado(String idproducto, String idcentro, String cantidad, String idempleado) {
		presenter.saveStockSolicitado(idproducto, idcentro, cantidad, idempleado);
	}
}
