package com.example.moventaa;

import java.io.IOException;
import java.util.List;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.app.SearchManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint;
import com.example.moventaa.bean.moventaaendpoint.model.CollectionResponseProducto;
import com.example.moventaa.bean.moventaaendpoint.model.Producto;
import com.example.moventaa.utils.ImageDownloader;
import com.example.moventaa.utils.Utils;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;

/**
 * Activity que implementa busqueda de 
 * productos
 * 
 * @author mmartinezced
 */
public class ProductoSearchActivity extends ListActivity
										implements OnCancelListener{

	protected static ProgressDialog myProgressDialog = null;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_productos_list);

	    // Get the intent, verify the action and get the query
	    Intent intent = getIntent();
	    if (Intent.ACTION_SEARCH.equals(intent.getAction())) {
	      String query = intent.getStringExtra(SearchManager.QUERY);
	      doMySearch(query);
	    }
	}
	
	/**
	 * Realiza una busqueda de producto por nombre
	 */
	private void doMySearch(String query){
		
		ProductoTaskRetriever productSearch = new ProductoTaskRetriever();
		productSearch.execute(query);
	}
	
	/**
	 * Carga los datos en la view
	 * @param result
	 */
	private void filldataView(CollectionResponseProducto result){
		if(result instanceof CollectionResponseProducto){
			List<Producto> items = result.getItems();
			ProductosRowAdapter adapter = new ProductosRowAdapter(this,items);
	        setListAdapter(adapter);
	        adapter.notifyDataSetChanged();
		}
	}

	
	protected void onListItemClick(ListView l, View v, int position, long id) {
		// TODO Auto-generated method stub
	   super.onListItemClick(l, v, position, id);

   	   myProgressDialog = ProgressDialog.show(this, 
											  "",
											  getResources().getString(R.string.wait), 
											  true,
											  true,
											  this);
   	   
   	   Intent myIntent = new Intent(this, ProductoInfoActivity.class);
   	   myIntent.putExtra(ProductoInfoActivity.INTENT_EXTRA,
   			   			 (String)v.getTag(R.string.TAGPRODROW));
   	   startActivity(myIntent);
	}	   
	
	
	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * (e.g., a store)
	 */
	private class ProductoTaskRetriever extends AsyncTask<String, Void, CollectionResponseProducto> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CollectionResponseProducto doInBackground(String... params) {
			Moventaaendpoint.Builder builder = new Moventaaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Moventaaendpoint endpoint = builder.build();
			try {

				return endpoint.findProductoByName().setName(params[0]).execute();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CollectionResponseProducto result) {
       	  super.onPostExecute(result);
       	  
		  if (result != null) {
			  filldataView(result);
		  }else{
			  Toast.makeText(getBaseContext(),
						"No se han encontrado datos...!!",
						Toast.LENGTH_LONG).show();
		  }
	   }
	} 
	
	
   /**
    * Adaptador para el listado de productos sobre la row
    * 
    * @author mmartinezced
    */
   public class ProductosRowAdapter extends ArrayAdapter<Producto> {
		 
        private final Context context;
        private final List<Producto> itemsArrayList;
        
        private ImageDownloader imagedownloader = new ImageDownloader();
        
        public ProductosRowAdapter(Context context, List<Producto> list) {
            super(context, R.layout.producto_search_row,list);
            this.context = context;
            this.itemsArrayList = list;
        }
        
        @Override
        public View getView(final int position, View convertView, ViewGroup parent) {
        	
            //Create inflater 
            LayoutInflater inflater = (LayoutInflater) context
                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            
            View rowView = inflater.inflate(R.layout.producto_search_row, parent, false);
            
            TextView title = (TextView) rowView.findViewById(R.id.prodsearchtitle);
            title.setText(itemsArrayList.get(position).getNombre());
            
            rowView.setTag(R.string.TAGPRODROW,itemsArrayList.get(position).getKey().getName());
            
            
    		ImageView imageView = (ImageView) rowView.findViewById(R.id.prodsearch_image);
            imagedownloader.download(Utils.getProductsUrlImage(itemsArrayList.get(position).getCodigoBR()),
            		                imageView);
            return rowView;
        }
	}


	@Override
	public void onCancel(DialogInterface dialog) {
		// TODO Auto-generated method stub
		
	}
   	
}
