package com.example.moventaa.presenter;

import java.io.IOException;

import android.os.AsyncTask;

import com.example.moventaa.CloudEndpointUtils;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint.LocalidadesByProvicia;
import com.example.moventaa.bean.moventaaendpoint.model.CollectionResponseLocalidad;
import com.example.moventaa.bean.moventaaendpoint.model.CollectionResponseProvincia;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;


/**
 * Patron de diseno MVP
 * 
 * Presenter para la busqueda de centros, provincias y localidades
 * 
 * @author mmartinezced
 *
 */
public class BusquedaCentrosPresenter {

	private IBusquedaCentrosView busquedaCentrosView;
	
	public BusquedaCentrosPresenter(IBusquedaCentrosView busquedaCentrosView){
		this.busquedaCentrosView = busquedaCentrosView;
	}
	
	
	public void searchProvincias(){
		ComboDataTaskRetriever task = new ComboDataTaskRetriever();
		task.execute();
	}
	
	public void searchLocalidades(String idLocalidad){
		LocalidadesDataTaskRetriever task = new LocalidadesDataTaskRetriever();
		task.execute(idLocalidad);
	}
	
	
	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * (e.g., a store)
	 */
	private class ComboDataTaskRetriever extends AsyncTask<Void, Void, CollectionResponseProvincia> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CollectionResponseProvincia doInBackground(Void... params) {
			
			
			Moventaaendpoint.Builder builder = new Moventaaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);

			builder = CloudEndpointUtils.updateBuilder(builder);

			Moventaaendpoint endpoint = builder.build();

			try {
				CollectionResponseProvincia lst = endpoint.listProvincia().execute();
				return lst;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CollectionResponseProvincia result) {
       	   super.onPostExecute(result);
		  if (result != null || result.getItems() != null || result.getItems().size() > 0) {
			  busquedaCentrosView.loadSpinnerProvincias(result.getItems());
		  }    
	   }
	}  
	
	
	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * (e.g., a store)
	 */
	private class LocalidadesDataTaskRetriever extends AsyncTask<String, Void, CollectionResponseLocalidad> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CollectionResponseLocalidad doInBackground(String... params) {
			
			Moventaaendpoint.Builder builder = new Moventaaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Moventaaendpoint endpoint = builder.build();

			try {
				LocalidadesByProvicia lbpo = endpoint.localidadesByProvicia().setIdProvincia(params[0]);
				CollectionResponseLocalidad lst = lbpo.execute();
				
				return lst;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CollectionResponseLocalidad result) {
       	   super.onPostExecute(result);
		  if (result != null && result.getItems() != null && result.getItems().size() > 0) {
			  busquedaCentrosView.loadSpinnerLocalidades(result.getItems());
		  }    
	   }
	}  
	
	
	
	
}
