package com.example.moventaa.presenter;

import java.io.IOException;

import android.app.DownloadManager;
import android.content.Context;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Environment;

import com.example.moventaa.CloudEndpointUtils;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint;
import com.example.moventaa.bean.moventaaendpoint.model.CollectionResponseNominas;
import com.example.moventaa.utils.Utils;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;

/**
 * Clase MVP para ralizar los contratos dedicados 
 * a la obtencion de nominas
 * 
 * @author mmartinezced
 *
 */
public class NominasPresenter {

	INominasView nominasview;	
	
	public NominasPresenter(INominasView nominasview){
		this.nominasview = nominasview;
	}
	
	public void searchNominas(String idempleado){
		ListDataTaskRetriever task = new ListDataTaskRetriever();
		task.execute(idempleado);
	}
	
	
	public void descargaNomina(String idempleado,String mes, DownloadManager manager){
		
		String url = Utils.getNominaURL(mes,idempleado);
		DownloadManager.Request request = new DownloadManager.Request(Uri.parse(url));
		request.setDescription("Descargando nomina");
		request.setTitle("Moventaa");
		// in order for this if to run, you must use the android 3.2 to compile your app
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
		    request.allowScanningByMediaScanner();
		    request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED);
		}
		request.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, "nomina-" + mes + ".pdf");


		manager.enqueue(request);
	}
	

	/**
	 * AsyncTask for calling Mobile Assistant API for checking into a place
	 * (e.g., a store)
	 */
	private class ListDataTaskRetriever extends AsyncTask<String, Void, CollectionResponseNominas> {

		/**
		 * Calls appropriate CloudEndpoint to indicate that user checked into a
		 * place.
		 *
		 * @param params
		 *            the place where the user is checking in.
		 */
		@Override
		protected CollectionResponseNominas doInBackground(String... params) {
			Moventaaendpoint.Builder builder = new Moventaaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Moventaaendpoint endpoint = builder.build();
			try {
				
				return endpoint.listNominasEmpleado(params[0]).execute();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return null;
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(CollectionResponseNominas result) {
       	  super.onPostExecute(result);
       	  nominasview.hideProgress();
		  if (result != null && result.getItems() != null && result.getItems().size() > 0) {
			  nominasview.fillRowsList(result.getItems());
		  }
	   }
	}
	
}
