package com.example.moventaa.presenter;

import java.io.IOException;

import android.os.AsyncTask;

import com.example.moventaa.CloudEndpointUtils;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint;
import com.example.moventaa.bean.moventaaendpoint.Moventaaendpoint.AnadirSolicitudStock;
import com.example.moventaa.bean.moventaaendpoint.model.ProductoInfo;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.json.jackson2.JacksonFactory;

/**
 * Patron de diseno MVP
 * Presenter de productos se encarga de llamar a
 * los datos y enviarlos a la capa de vista
 * 
 * @author mmartinezced
 *
 */
public class ProductoInfoPresenter {

	private IProductoInfoView productoView;
    //private LoginInteractor loginInteractor;
	
	
	public ProductoInfoPresenter(IProductoInfoView view){
		this.productoView = view;
	}
	
	
	public void loadProducto(String idproducto, String idcentro){
		ProductoTaskRetriever producto = new ProductoTaskRetriever();
		producto.execute("find",idproducto, idcentro);
    }
	
	public void loadProductoByBarCode(String barcode, String idcentro){
		ProductoTaskRetriever producto = new ProductoTaskRetriever();
		producto.execute("barcode",barcode, idcentro);
	}
	
	
	public void saveStockSolicitado(String idproducto, String idcentro, String stock, String idempleado){
		ProductoTaskRetriever producto = new ProductoTaskRetriever();
		producto.execute("stock",idproducto, idcentro, stock, idempleado);
	}
	
	/**
	 * AsyncTask for calling Mobile Assistant API
	 * 
	 *  
	 * (e.g., a store)
	 */
	private class ProductoTaskRetriever extends AsyncTask<String, Void, Object> {

		protected void onPreExecute() {
			productoView.showProgress();
		}
		
		/**
		 * Calls appropriate CloudEndpoint 
		 * @param params
		 */
		@Override
		protected Object doInBackground(String... params) {
			Moventaaendpoint.Builder builder = new Moventaaendpoint.Builder(
					AndroidHttp.newCompatibleTransport(), new JacksonFactory(),
					null);
			builder = CloudEndpointUtils.updateBuilder(builder);
			Moventaaendpoint endpoint = builder.build();
			Object objectreturn = null;
			try {
				if("find".equals(params[0])){
					//Busqueda por producto id
					objectreturn = endpoint.findInfoProductoById(params[1],params[2]).execute();
					return objectreturn;
				}else if("barcode".equals(params[0])){
					//Busqueda por barcode
					objectreturn = endpoint.findInfoProductoByBarcode(params[1],params[2]).execute();
					return objectreturn;
				}else{
					//Asigna stock al producto
					AnadirSolicitudStock endp = endpoint.anadirSolicitudStock();
					endp.setIdProducto(params[1]);
					endp.setIdCentro(params[2]);
					endp.setCantidad(params[3]);
					endp.setIdEmpleado(params[4]);
					endp.execute();
					objectreturn = new String("stock");
					return objectreturn;
				}	
			} catch (IOException e) {
				e.printStackTrace();
			}finally{
				return objectreturn;
			}
			
	   }
		
	   @SuppressWarnings("null")
	   protected void onPostExecute(Object result) {
		  super.onPostExecute(result);
       	  if(result instanceof ProductoInfo){
       		  productoView.showProductoDataDetails((ProductoInfo)result);
       		  productoView.hideProgress();
       	  }else if ("stock".equals(result)){
       		  productoView.hideProgress();
       		  productoView.showSolicitudStockEnviada();
       	  }
	   }
	}  
	
}
